(function($){
	$.fn.autom8 = function(options){
		var self = this;
		self.opts = $.extend(true, {}, $.fn.autom8.defaults, options);

		var key = {
			left: 37,
			up: 38,
			right: 39,
			down: 40,
			del: 46,
			tab: 9,
			enter: 13,
			esc: 27,
			space: 32,
			comma: 188,
			pageup: 33,
			pagedn: 34,
			backspace: 8
		};

		//initialize the dictionary
		self.arr = null; //dictionary var
		switch (typeof self.opts.dictionary) {
			case "string": //dictionary is an external file to load
				$.get(self.opts.dictionary, false, function(text){
					self.arr = text;
				});
				break;
			case "object": //dictionary is an array of strings to load
				self.arr = self.opts.dictionary.join("\n");
				break;
			default:
				throw "Error - No dictionary defined.";
		}
		self.arr = "\n" + self.arr; //need a newline at the start so our regex doesnt miss the first word

		//default callback for showing the next page of suggestions and selecting the first suggestion. will not be called if pagination is disabled or if a callback is defined for the onShowNextPage option
		function show_next_page_default(options){
			var page = options.page; //current page number to show
			var pp = options.suggestionsPerPage;
			var m = options.suggestions; //all matches
			var s = $(options.suggestionsContainer); //suggestions container
			var wrapIn = options.wrapIn;
			var selectedClass = options.selectedClass;

			//list new suggestions
			s.html('<' + wrapIn + '>' + m.slice(0 + page * pp, pp + (page * pp)).join('</' + wrapIn + '><' + wrapIn + '>') + '</' + wrapIn + '>');

			//select the first suggestion in the new list
			s.find(wrapIn).eq(0).addClass(selectedClass).show();
		}
		//default callback for showing the previous page of suggestions and selecting the last suggestion. will not be called if pagination is disabled or if a callback is defined for the onShowPrevPage option
		function show_prev_page_default(options){
			var page = options.page; //current page number to show
			var pp = options.suggestionsPerPage;
			var m = options.suggestions; //all matches
			var s = $(options.suggestionsContainer); //suggestions container
			var wrapIn = options.wrapIn;
			var selectedClass = options.selectedClass;

			//list new suggestions
			s.html('<' + wrapIn + '>' + m.slice(0 + page * pp, pp + (page * pp)).join('</' + wrapIn + '><' + wrapIn + '>') + '</' + wrapIn + '>');

			//select the last suggestion in the new list
			s.find(wrapIn).eq(pp).addClass(selectedClass).show();
		}

		//default callback function for showing suggestions. will not be called if a callback is defined for the onShowSuggestions option
		function show_suggestions_default(options){
			//listing out other options for reference
			//options.word
			//options.animation
			//options.animationSpeed
			//options.disableAnimations
			var selectSuggestion = options.selectSuggestion; //callback for selecting the current selection (the item with the 'selectedClass' on it)
			var sc = options.scope;
			var s = $(options.suggestionsContainer);
			var m = options.suggestions;
			var pp = options.suggestionsPerPage;
			var wrapIn = options.wrapSuggestionsIn;
			var selectedClass = options.selectedSuggestionClass;

			if (pp > 0) { //pagination on suggestions
				s.html('<' + wrapIn + ' class="' + selectedClass + '">' + m.slice(0, pp).join('</' + wrapIn + '><' + wrapIn + '>') + '</' + wrapIn + '>');
			}
			else { //no patination on suggestions
				s.html('<' + wrapIn + ' class="' + selectedClass + '">' + m.join('</' + wrapIn + '><' + wrapIn + '>') + '</' + wrapIn + '>');
			}

			//mouse interaction - hover and click. need to use live in case of pagination
			$(options.suggestionsContainer+' '+wrapIn)
				.die('click,mouseover')
				.live('click',selectSuggestion)
				.live('mouseover', function(){
					$('.' + selectedClass, s).removeClass(selectedClass);
					$(this).addClass(selectedClass);
				});


			//show the suggestions
			if (options.disableAnimations) {
				s.stop().show().css('opacity', 1);
			}
			else {
				if (options.animation) { //animate
					switch (options.animation) {
						case 'show':
							s.stop().css("opacity", 0).show(options.animationSpeed);
							break;
						case 'slide':
							s.stop().css({
								height: 0,
								opacity: 1
							}).slideDown(options.animationSpeed);
							break;
						//case 'fade' - default
						default:
							s.stop().css("opacity", 0).show().fadeTo(options.animationSpeed, 1);
							break;
					}
				}
				else { //fade - default
					s.stop().css("opacity", 0).fadeTo(options.animationSpeed, 1);
				}
			}
		}

		//default callback function for hiding suggestions. will not be called if a callback is defined for the onHideSuggestions option
		function hide_suggestions_default(options)
		{
			var s = $(options.suggestionsContainer);
			s.hide().empty();
		}

		return self.each(function(i, sc){
			//sc for 'scope'
			sc.c = 0; // caret
			sc.w = ''; // word
			sc.st = 0; // word start
			sc.en = 0; // word end
			sc.l = self.opts.minCharsToMatch; // min length
			sc.el = $(sc); // element
			sc.v = sc.el.val(); // val
			sc.s = $(self.opts.onShowSuggestionsOptions.suggestionsContainer); // container(s) to populate with the suggestions
			sc.m = []; // matches
			sc.pp = self.opts.onShowSuggestionsOptions.suggestionsPerPage; // suggestions per page
			sc.page = 0; // page in suggest list
			sc.autofilled = false; //true if autofilled
			sc.autofillen = true; // false when user disables
			sc.autofillst = -1; // track starting position for autofill
			//either calls the default, internal show_suggestions_default() function or the onShowSuggestionsOptions callback option
			sc.show_suggestions = function(disableAnimations)
			{
				var suggestionsVisible = sc.s.is(':visible');
				var isNewWord = sc.get_word(); //returns bool
				if(!isNewWord && suggestionsVisible) return false;

				var matchesFound = sc.get_matches(); //returns bool

				if (!(sc.m.length > 0 && (sc.m.length > 1 || ($.trim(sc.m[0]) != sc.w)))) { //hide suggestions
					if (suggestionsVisible) sc.hide_suggestions();
					return false;
				}

				//if the suggestions list is already visible, force disable animations
				if (suggestionsVisible) {
					disableAnimations = true;
				}

				//callback options
				var callbackOpts = $.extend({}, {
					selectSuggestion: sc.select_suggestion, //callback for selecting the current selection (the item with the 'selectedClass' on it)
					element: sc.el, //the element we're typing within. this is a jquery object
					word: sc.w, //the current partial word we're finding suggestions for
					suggestions: sc.m, //array of matching suggestions
					disableAnimations: disableAnimations
				}, self.opts.onShowSuggestionsOptions); //add user options

				if (self.opts.onShowSuggestions) { //callback custom method to show suggestions
					self.opts.onShowSuggestions(callbackOpts);
				}
				else { //use internal method to show suggestions
					show_suggestions_default(callbackOpts);
				}
        
        sc.autofill();
			}

			//either calls the default, internal hide_suggestions_default() function or the onHideSuggestionsOptions callback option
			sc.hide_suggestions = function(disableAnimations){
				sc.autofilled = false;

				if (sc.s.is(':visible')) { //only hide if the suggestions are visible
					//callback options
					var callbackOpts = $.extend({}, {
						selectSuggestion: sc.select_suggestion, //callback for selecting the current selection (the item with the 'selectedClass' on it)
						element: sc.el, //the element we're typing within. this is a jquery object
						word: sc.w, //the current partial word we're finding suggestions for
						suggestions: sc.m, //array of matching suggestions
						suggestionsContainer: self.opts.onShowSuggestionsOptions.suggestionsContainer,
						disableAnimations: disableAnimations
					}, self.opts.onHideSuggestionsOptions); //add user options

					if (self.opts.onHideSuggestions) { //callback custom method to show suggestions
						self.opts.onHideSuggestions(callbackOpts);
					}
					else { //use internal method to show suggestions
						hide_suggestions_default(callbackOpts);
					}
				}
			}

			//auto inserts the current suggestion as a hint, but doesn't accept until enter is pressed
			sc.autofill = function(){

        if(!sc.autofillen) return false;
        var ast = (sc.autofillst>=0)?sc.autofillst:sc.en;
        
				var f = sc.get_selected_suggestion();
				if (!f) return false; //no suggestion selected
				var ftxt = $.trim(f.text());

				var r = (ast == sc.v.length) ? ftxt + ' ' : ftxt;
				var left = sc.v.substring(0, sc.st);
				var right = (sc.autofilled)?sc.v.substr(sc.c.end):right = sc.v.substr(ast);
				
				sc.el.val(left + r + right);
				sc.el.caret(ast, ast + r.length- sc.w.length);
        
        if(console) console.log(ast+' - '+(ast, ast + r.length- sc.w.length));
        
        //sc.autofillst = sc.en;
				sc.autofilled = true;
			}

			sc.remove_autofill = function(){
				if(!sc.autofilled) return;
				var cStart = sc.c.start;
				var left = sc.v.substring(0, cStart);
				var right = sc.v.substr(sc.c.end);
				sc.el.val(left + right);

				sc.el.caret(cStart);
			}
			
			sc.handle_key = function(k)
			{
				if($.inArray(k, [key['enter'], key['space'], key['down'], key['tab'] ])>=0)
          sc.autofillen = true; // activate autofill again
				if ($.inArray(k, [key['right'], key['left']]) >= 0)
          sc.autofillen = false;
				
				
				if ($.inArray(k, [key['right'], key['esc']]) >= 0) {
					return true; //ignore these special chars here
				}
				else 
				if ($.inArray(k, [key['backspace'], key['esc'], key['del'] ]) >=0)
				{
					if(sc.autofilled)
					{ sc.remove_autofill(); sc.autofillen = false; sc.autofilled = false; }
					return true;
				}
				// ADD SC.ST CHANGE CHECK, NOT WORKING
				else if($.inArray(k,[key['enter'],key['space'],key['up'],key['down'],key['tab']])>=0)
				{					
					if      (sc.autofilled && k === key['enter']){ var result = sc.select_suggestion(); } 
          else if (k === key['up'])   {	var result = sc.move_suggestion('prev'); }
          else if (k === key['down'] || k === key['tab']) { var result = sc.move_suggestion('next'); }

          // if selected, return false to disable key
          return !result;
				}
			}

			//invoked anytime the selection within the suggestion box has been changed
			sc.selection_changed = function(){
				sc.autofill();

				if (self.opts.onChangeSuggestion) {
					var f = sc.get_selected_suggestion();
					if (!f) return false; //no suggestion selected

					var selectedSuggestion = $.trim(f.text());

					var options = {
						element: sc.el, //the element we're typing within. this is a jquery object
						word: sc.w, //the current partial word we're finding suggestions for
						caretPosition: sc.c, //current caret position within the element that we are typing
						selectedSuggestion: selectedSuggestion //the newly selected word within the suggestion bank
					};
					self.opts.onChangeSuggestion(options);
				}
			}

			sc.select_suggestion = function(){
				var f = sc.get_selected_suggestion();
				if (!f) return false; //no suggestion selected

				if(sc.autofilled){
					sc.hide_suggestions();
					sc.el.caret(sc.st + f.text().length);
					sc.autofillen = true; // enable autofill again					
					return true;
				}

				var r = (sc.en == sc.v.length) ? $.trim(f.text()) + ' ' : $.trim(f.text());
				var left = sc.v.substring(0, sc.st);
				var right = sc.v.substr(sc.en);

				sc.el.val(left + r + right);
				sc.hide_suggestions();
				sc.el.caret(sc.st + f.text().length);

				if (self.opts.onSelectSuggestion) {
					var selectedSuggestion = $.trim(f.text());

					var options = {
						element: sc.el, //the element we're typing within. this is a jquery object
						caretPosition: sc.el.val().length, //current caret position within the element that we are typing
						selectedSuggestion: selectedSuggestion //the newly selected word within the suggestion bank
					};
					self.opts.onSelectSuggestion(options);
				}

				return true;
			}

			sc.get_selected_suggestion = function(){
				var wrapIn = self.opts.onShowSuggestionsOptions.wrapSuggestionsIn;
				var selectedClass = self.opts.onShowSuggestionsOptions.selectedSuggestionClass;
				var f = sc.s.find(wrapIn + '.' + selectedClass);
				if (f.length > 0) {
					return f;
				}
				else return false;
			}

			//selects the next/prev item in the suggestion list
			//f is the current selected suggestion, before the next one is selected
			sc.move_suggestion = function(direction)
			{
				if (sc.m.length <= 0) {
					return false; //no suggestions to show
				}

				var f = sc.get_selected_suggestion();
				if (!f) return false; //no suggestion selected

				var wrapIn = self.opts.onShowSuggestionsOptions.wrapSuggestionsIn;
				var selectedClass = self.opts.onShowSuggestionsOptions.selectedSuggestionClass;
				var selectedSuggestion = '';

				if (direction == 'next') {
					if (sc.pp > 0 && f.next().length <= 0) { //pagination on suggestions
						//update the page number
						sc.page = (sc.pp + (sc.pp * sc.page) < sc.m.length) ? sc.page + 1 : 0;

						var options = {
							page: sc.page, //current page number
							suggestionsPerPage: sc.pp, //suggestions per page
							suggestions: sc.m, //all matches
							suggestionsContainer: self.opts.onShowSuggestionsOptions.suggestionsContainer, //suggestions container
							wrapIn: self.opts.onShowSuggestionsOptions.wrapSuggestionsIn,
							selectedClass: self.opts.onShowSuggestionsOptions.selectedSuggestionClass
						};

						if(self.opts.onShowNextPage){ //custom callback in the options
							self.opts.onShowNextPage(options);
						}
						else{ //use the default callbak
							show_next_page_default(options);
						}
					}

					if (f.next().length > 0) {
						selectedSuggestion = f.removeClass(selectedClass).next().addClass(selectedClass).text();
					}
					else {
						f.removeClass(selectedClass);
						selectedSuggestion = sc.s.find(wrapIn + ':first').addClass(selectedClass).text();
					}
				}
				else if (direction == 'prev') {
					if (sc.pp > 0 && f.prev().length <= 0) { //pagination on suggestions
						//update the page number
						sc.page = (((sc.pp * sc.page) - 1) > 0) ? sc.page - 1 : (Math.ceil(sc.m.length / sc.pp) - 1);

						var options = {
							page: sc.page, //current page number
							suggestionsPerPage: sc.pp, //suggestions per page
							suggestions: sc.m, //all matches
							suggestionsContainer: self.opts.onShowSuggestionsOptions.suggestionsContainer, //suggestions container
							wrapIn: self.opts.onShowSuggestionsOptions.wrapSuggestionsIn,
							selectedClass: self.opts.onShowSuggestionsOptions.selectedSuggestionClass
						};

						if(self.opts.onShowPrevPage){ //custom callback in the options
							self.opts.onShowPrevPage(options);
						}
						else{ //use the default callbak
							show_prev_page_default(options);
						}
					}

					if (f.prev().length > 0) {
						selectedSuggestion = f.removeClass(selectedClass).prev().addClass(selectedClass).text();
					}
					else {
						f.removeClass(selectedClass);
						selectedSuggestion = sc.s.find(wrapIn + ':last').addClass(selectedClass).text();
					}
				}

				sc.selection_changed(); //fire the selection_changed event
				return true;
			}

			//sets sc.m. returns true if a new word is found, false if the same word is found as previous
			sc.get_word = function(){
				// start, end
				var s1 = sc.v.substring(0, sc.c.end).search(/\s\S*?$/);
				var e1 = sc.v.substring(sc.c.end, sc.v.length).search(/\s/);
				sc.st = (s1 >= 0) ? s1 + 1 : 0;
				sc.en = (e1 >= 0) ? e1 + sc.c.end : sc.v.length;
				var curWord = sc.v.substring(sc.st, sc.en);

				$('#debug').html('<table width="100%"><tr><td width="45">caret</td><td width="45">start</td><td width="45">end</td><td>word</td></tr><tr><td>' + sc.c.start + ':' + sc.c.end + '</td><td>' + s1 + ':' + sc.st + '</td><td>' + e1 + ':' + sc.en + '</td><td>' + sc.w + '</td></tr></table>').show();

				if (curWord != sc.w) { //new word
					sc.w = curWord;
					return true;
				}
				else { //same as current word
					return false;
				}
			}

			//sets sc.w. returns true matches are available, false otherwise
			sc.get_matches = function(){
				if (sc.w.length >= sc.l) {
					var matches = [];

					var re = new RegExp('\n' + RegExp.escape(sc.w.toLowerCase()) + '.*', 'gi');
					matches = self.arr.match(re);

					if (matches) {
						sc.m = matches;
						return true;
					}
				}
				sc.m = false;
				return false;
			}

			sc.keydown = function(e){
				var k = e.keyCode ? e.keyCode : e.which; // keycode
				sc.c = sc.el.caret(); // caret
				sc.v = sc.el.val(); // val
				//$('#debug2').append(' .'+sc.autofilled.toString().substring(0,1)+'-'+sc.autofillen.toString().substring(0,1)+'. ');
				return sc.handle_key(k);
				//return true;
			}
			sc.el.bind(($.browser.mozilla ? 'keypress' : 'keydown') + '.autocomplete', sc.keydown);
			
      sc.el.bind("click", function(e) {
        sc.hide_suggestions();
      });

			sc.keyup = function(e){
				var k = e.keyCode ? e.keyCode : e.which; // keycode
				sc.c = sc.el.caret(); // caret
				sc.v = sc.el.val(); // val
				if (sc.v.length < sc.l) {
					sc.hide_suggestions();
					return true;
				}

				var disableAnimations = false;
				if (k === key['backspace'] || k === key['space']) {
					disableAnimations = true;
					if (k === key['space']) {
						if (sc.c === sc.v.length) {
							sc.el.caret(sc.v.length);
						}
					}
					else if (k !== key['backspace']) {
						disableAnimations = false;
					}
					sc.hide_suggestions(disableAnimations);
				}
				//ignore here. selects next or prev word
				else if ($.inArray(k,[key['down'],key['up'],key['tab']])>=0)
				{
					return true;
				}
				else if (k === key['esc']) {
					sc.hide_suggestions();
					return false;
				}

				sc.show_suggestions(disableAnimations);

				return true;
			}
			sc.el.bind("keyup", sc.keyup);


			// Just a quick placeholder for Regexp escape function
			// http://simonwillison.net/2006/Jan/20/escape/
			RegExp.escape = function(text){
				if (!arguments.callee.sRE) {
					var specials = ['/', '.', '*', '+', '?', '|', '(', ')', '[', ']', '{', '}', '\\'];
					arguments.callee.sRE = new RegExp('(\\' + specials.join('|\\') + ')', 'g');
				}
				return text.replace(arguments.callee.sRE, '\\$1');
			}
		});
	};

	//DEFAULT OPTOINS
	$.fn.autom8.defaults = {
		dictionary: ['apple', 'application', 'apply', 'box', 'book', 'string', 'strap', 'strip'], //an array of words to match or a URL of a dictonary file to load with words separated by newlines
		minCharsToMatch: 2, //the minimum number of letters before a match is suggested
		onShowSuggestions: null, //custom callback to show the suggestions. if null, uses internal method
		onShowSuggestionsOptions: { //options passed to the show_suggestions callback
			suggestionsContainer: '#suggestions', //selector for the container(s) to populate with suggestions
			wrapSuggestionsIn: 'span', //each suggestion is wrapped in this element type
			selectedSuggestionClass: 's', //class added to the selected suggestion
			suggestionsPerPage: 7, // <=0 means unlimited, >0 does patination automatically (with unlimited, you can do something in CSS like overflow-y: scroll; height: 150px; on your suggestionsContainer to use a scrollbar)
			animation: 'fade', //can be 'fade', 'slide', 'show', or null (set to 'show' with a speed of null for no animation)
			animationSpeed: 200 //milliseconds
		},
		onHideSuggestions: null, //custom callback to hide the suggestions. if null, uses internal method
		onHideSuggestionsOptions: { //options passed to the show_suggestions callback
			animation: 'fade', //can be 'fade', 'slide', 'hide', or null (set to 'hide' with a speed of null for no animation)
			animationSpeed: 200 //milliseconds
		},
		onShowNextPage: null, //custom callback to handle displaying the 'next page' when pagination is used (suggestionsPerPage > 0). should also select the first item in the updated list
		onShowPrevPage: null, //custom callback to handle displaying the 'prevoius page' when pagination is used (suggestionsPerPage > 0). should also select the last item in the updated list
		onSelectSuggestion: null, //callback when a selection has been made
		onChangeSuggestion: null //callback when a suggestion selection has been changed
	};
})(jQuery);