if (typeof console === 'undefined') {
    console = {log: function() {return;} };
}

var pw = {w: 800, h: 400};

// Name of the object that will namespace our protovis visualization
/*
    TODO: Making all of our generated objects properties of a container object is a good idea
    (at least have as an option), since it allows protowiz visualizations to safely be dropped
    into pages with existing javascript.  However, it's making the code expansion too fiddly
    to deal with for this first version.  It also makes the code harder to read & understand
    for someone new to protovis.
*/ 
// pw.name = "theVis";

// Takes a jQuery select tag, an array used to populate its options,
// a selected option value, and functions that transform the array entries
// into an option value and option text.  Appends the generated option tags
// to the select tag, and returns true if the specified selected option was found
// in the list, or false otherwise.
pw.appendOptions = function(select, optionArray, selectedOpt, valueFunc, textFunc) {
    selectedOpt = selectedOpt || '';
    valueFunc = valueFunc || function(i, v) {return i};
    textFunc = textFunc || function(i, v) {return v};
    
    var foundSelected = false,
        selectedText = '',
        optionText = '';
    
    for (var i=0; i<optionArray.length; i++) {
        var optionValue = valueFunc(i, optionArray[i]).toString(),
            optionText = textFunc(i, optionArray[i]).toString(),
            s = selectedOpt.toString();
        // If the current option corresponds to this property's current value,
        // select it and note that we found the existing dataSet on the list
        if (optionValue === s) {
            foundSelected = true;
            selectedText = ' selected="selected"';
        } else {
            selectedText = '';
        }           
        select.append('<option value="' + optionValue + '"' + selectedText + '>' + optionText + '</option>');
    }
    
    return foundSelected;
}

// This "forces" the combo box to take a certain value
// If the value is already in the pick list, we just select it.
// If it's not available, we change the editable option to the new value
pw.setSimpleCombo = function(select, value) {
	if (!(select instanceof jQuery)) {
	    select = $(select);
	}
	var isSet = false;
	select.children('option').each(function() {
		var selected = !isSet && ($.trim(this.value) === $.trim(value) ||
			$.trim(this.text) === $.trim(value));
		isSet = isSet || selected;
		this.selected = selected;
	});
	if (!isSet) {
	    select.children('option').eq(0).attr('value', value).text(value).selected = "selected";
	}			    
}

pw.anchorSelect = function() {
    var aSelect = $('<select/>', {'class': 'anchorSelect'});
    $.each(["'top'", "'right'", "'bottom'", "'left'", "'center'"], function(i, dir) {
        aSelect.append('<option value="' + dir + '">' + dir + '</option>');
    });
    return aSelect.simpleCombo();
}

// attach a color picker to the passed-in element
pw.attachPicker = function(el) {
	if (!(el instanceof jQuery)) {
	    el = $(el);
	}
    var span = $('<span class="picker">&nbsp;&nbsp;&nbsp;</span>');
    el.after(span);
    var c = new jscolor.color(el[0], {
        required: false,
        adjust: false,
        styleElement: span[0],
        hash: true,
        pickerPosition: 'top',
        caps: false
    });
    return c;
}

// Name of the root panel of our visualization
// TODO: make this an actual user-specified panel with the special property that it's always at layer 0 and it can't be deleted (?)
pw.rootName = "root";

// list of marks protowiz supports
pw.supportedMarks = ['Mark', 'Area', 'Bar', 'Dot', /*'Image', */ 'Label', 'Line', 'Panel', 'Rule', 'Wedge'];

// object containing a sub-object for each mark
// sub-objects that have a property (with value=true) for each property that the mark supports
pw.properties = {};
$.each(pw.supportedMarks, function(i, v) {
    pw.properties[v] = $.extend({}, pv[v].prototype.properties);
});

// Add our custom properties to the basic Mark list
pw.properties.Mark = $.extend(pw.properties.Mark, {
    name: true,
    type: true,
    parent: true
});

// collapse two properties that protovis treats separately but that we treat as one
pw.collapse = function(type, find, replace) {
    var obj = pw.properties[type];
    if (typeof find === 'string') {
        find = [find];
    }
    $.each(find, function(i, v) {
        delete obj[v];
    });
    obj[replace] = true;
}

pw.collapse('Mark', ['left', 'right'], 'x');
pw.collapse('Mark', ['top', 'bottom'], 'y');
pw.collapse('Wedge', ['startAngle', 'endAngle'], 'angle');
// "radius" is collapsed into "size"
delete pw.properties.Dot.radius;

pw.properties.Rule.tickLabel = true;

// An ordered list of all properties supported by protowiz.  Note that this:
// - Includes some properties that are not on any protovis mark (e.g. "type")
// - Excludes some protovis properties (e.g. "events")
// - Will include some properties that are combinations of protovis properties
//   (e.g. "x" instead of "left" and "right")
pw.allMarkProperties = [
    "name", // custom
    "type", // custom
    "parent", // custom
	"children", // custom
    "data",    
    // (note: the comments in this list are just to give some intuition as to why properties
    // are grouped the way they are in the order, not to be an exact key to how the properties are implemented.
    // For instance, some "advanced" properties are also "boolean".)
    // Scale and field properties
    "x",
    "y",
    "size",
    "width",
    "height",
    "angle",
    "text",
    // Style properties
    "strokeStyle",
    "fillStyle",
    "textStyle",
    // List properties
    "interpolate",
    "shape",
    "lineJoin",
    // Boolean properties
    "segmented",
    "visible",
    // Miscellaneous
    "title",
    "lineWidth",
    "tension",
    "url",
    "imageWidth",
    "imageHeight",
    "font",
    "textAngle",
    "textAlign",
    "textBaseline",
    "textMargin",
    "textDecoration",
    "eccentricity",
    "innerRadius",
    "outerRadius",
    "tickLabel", // custom
    // Advanced properties
    "antialias",
    "cursor",
    "reverse",
    "textShadow"
];

pw.advancedProperties = {
    "antialias": true,
    "cursor": true,
    "reverse": true,
    "textShadow": true
};

pw.showAdvancedProperties = false;

// list of all active marks
pw.marks = [];

// Update all lists of marks that need to be updated in the UI
pw.marks.updateLists = function() {
    for (var i=1; i<this.length; i++) {
        this[i].layer = i;
        this[i].properties.parent.updateForm();
    }
	pw.layersUI.writeInheritanceToHeader();
}

// Add a new mark to the list, and make it aware of what
// layer it corresponds to (which is just its index
// in the marks array)
pw.marks.add = function(newMark) {
    this.push(newMark);
    newMark.layer = this.length - 1;
    this.updateLists();
}

// Remove a mark from the list
pw.marks.del = function(delMark) {
    var curProps, delProps, newParent;
    // if any mark had the deleted mark as a parent,
    // hand the child mark up to the deleted mark's parent
    // It would be more elegant (and slightly more performant)
    // if each Mark just knew who its children were, but that
    // seems like overkill at this point
    for (var i=1; i<this.length; i++) {
        curProps = this[i].properties;
        delParent = delMark.properties.parent();
        if (curProps.parent().parent.toString() === delMark.layer.toString()) {
            // Basically: if the child mark that's being assigned to a new parent
            // has its anchor set already, don't touch it (just set the parent 
            // layer number).  If the child doesn't have an anchor set yet,
            // use the deleted mark's anchor as well as it's parent layer number.
            // This is really just a reasonable guess as to what's best when
            // the child and parent anchors conflict; I don't think there's a right
            // answer.
            newParent = curProps.parent().anchor === '' ? delParent : delParent.parent;
            curProps.parent(newParent);
        }
    }    
    
    this.splice(delMark.layer, 1);
    this.updateLists();
}

pw.marks.getLayerForId = function(id) {
	for (var i=0; i < this.length; i++) {
		if (this[i].id === parseInt(id, 10)) {
			return this[i].layer;
		} 
	}
};

pw.marks.getIdForLayer = function(layer) {
	for (var i=0; i < this.length; i++) {
		if (this[i].layer === parseInt(layer, 10)) {
			return this[i].id;
		} 
	}
};

pw.marks.getObjectWithId = function(id) {
	for (var i=0; i < this.length; i++) {
		if (this[i].id === parseInt(id, 10)) {
			return this[i];
		}; 
	};
};

pw.marks.getObjectOnLayer = function(layer) {
	for (var i=0; i < this.length; i++) {
		if (this[i].layer === parseInt(layer, 10)) {
			return this[i];
		}; 
	};
};

pw.supportedScales = ["Scale", "linear", "log", "ordinal",/* "quantile",*/ "root"];

pw.allScaleProperties = [
    "name",
    "type",
    "kind",
    "data",
    "domain",
    "ordinalDomain",
    "range",
    "ordinalRange"
];

pw.commonScaleProperties = [
    "name",
    "type",
    "kind",
    "data"
];

// Stub out scale-type-specific properties for now
// (Later this will add the properties specific to each scale type)
$.each(pw.supportedScales, function(i ,v) {
    pw.properties[v] = {}; 
});

$.each(pw.commonScaleProperties, function(i, v) {
    pw.properties.Scale[v] = true;
});
$.each(["linear", "log", "root"], function(i, v) {
    pw.properties[v].domain = true;
    pw.properties[v].range = true;
});
pw.properties.ordinal.ordinalDomain = true;
pw.properties.ordinal.ordinalRange = true;

pw.scales = [];

pw.scales.add = function(s) {
    this.push(s);
    this.updateLists();
}

pw.scales.options = function(select, selectedOpt) {
    var nameFunc = function(i, v) {return v.properties.name(); };
    return pw.appendOptions(select, this, selectedOpt, null, nameFunc);
}

// Update all lists of scales
pw.scales.updateLists = function() {
    $.each(pw.marks, function(i, mark) {
        mark.updateScaleAndFieldProperties();
    });
    pw.layersUI.refreshScaleSelector();
}


pw.allDataSetProperties = ["name", "dataSet", "nest", "nest2"];
pw.properties.dataSet = {};
$.each(pw.allDataSetProperties, function(i, v) {
    pw.properties.dataSet[v] = true;
});

pw.dataSets = [];

pw.dataSets.add = function(s) {
    this.push(s);
}

// Update all lists of dataSets
pw.dataSets.updateLists = function() {
    for (var i=0; i<pw.marks.length; i++) {
        pw.marks[i].properties.data.updateForm();
    }
    for (var i=0; i<pw.scales.length; i++) {
        pw.scales[i].properties.data.updateForm();
    }
}

// Remove a dataSet from the list
pw.dataSets.del = function(delSet) {
    var delIndex = null;
    // Find the dataSet we're trying to delete in the array, and note its index
    for (var i=0; i<this.length; i++) {
        if (this[i] === delSet) {
            delIndex = i;
            break;
        }
    } 
    
    // Remove the dataSet from the list
    this.splice(delIndex, 1);
    
    for (var i=0; i<pw.marks.length; i++) {
        var m = pw.marks[i],
            oldData = parseInt(m.properties.data());
        // If a mark was using the deleted data, unset its data property
        if (oldData === delIndex) {
            m.properties.data('');
        // If a mark was using a dataSet with a higher index than the one being deleted,
        // decrement the index.  Keeping track of all these indexes feels pretty fragile.
        } else if (oldData > delIndex) {
            m.properties.data(m.properties.data() - 1);
        }
    }
    
    this.updateLists();
}

// Adds <option> tags to a passed-in select tag that enumerate currently existing dataSets
// Optionally, selectedOpt can be passed in, in which case the corresponding option tag
// will have its selected attribute set.  The function returns true if the selectedOpt
// was found in the existing dataSets, and false otherwise
pw.dataSets.options = function(select, selectedOpt) {
    var textFunc = function(i, v) {return v.properties.name()};
    return pw.appendOptions(select, this, selectedOpt, null, textFunc);
}

// takes a string, massages it into a vaild identifier,
// then makes sure it's not already in use.
// If it is in use, appends a unique id and tries again.
pw.uniqueName = function(candidate, fallback) {
    var newName = candidate.toName(fallback);

    // if the chosen name is already in use by another mark,
    // tack a unique id on the end and try again.
    for (var i=0; i<pw.marks.length; i++) {
        if (pw.marks[i].properties.name() === newName) {
            newName = pw.uniqueName(newName + pw.id());
            break;
        }
    }
    return newName;
}

/** @private Returns a locally-unique positive id. */
// Ripped off directly from protovis - just making a pw-specific version so our numbers can start at 1
pw.id = function() {
  var id = 1;
  return function() { return id++; };
}();

// Expand protowiz shorthand code into working protovis code
// This function assumes it's dealing with one line at a time
pw.expand = function(code) {
    if (typeof code === 'undefined') return '';
    code = code.toString();
    if (code === '') return '';
    
    // If code already starts with "function(", don't mess with it.
    // Otherwise, if it uses "d." or "d[" prepend a "function(d) "
    // If it uses "this." or "this[" prepend a "function() "
    // Note that this method is currently quite weak.
    // It will get a false positive from, e.g. "this." inside a string literal.
    // It will get a false negative if "d" or "this" are used
    // but are not followed by a "." or "[".
    // To get this 100% reliable would require something close to a full
    // javascript parser, but this should cover the vast majority
    // of real-world cases.
    var starts_with_function = /^function\(/;
    if (!starts_with_function.test(code)) {
        var has_d = /([^0-9A-Za-z]|^)d[\[.]/;
        var has_this = /([^0-9A-Za-z]|^)this[\[.]/;
        if (has_d.test(code)) {
            code = "function(d) " + code;
        } else if (has_this.test(code)) {
            code = "function() " + code;
        }
    }
    
    return code;
}

// pw.property constructs and returns a property object, which is a function internally
// referred to as "accessor".
// Calling accessor without a parameter gets the value of the property
// Calling accessor with a parameter sets the value of the property, then returns it
// The function also has its own public methods that
// set the visibility of its form and return protovis code based on its value
// So, let's say you have a property called myProp with name set to "myProp"
// and value set to "foo":
// myProp() will return "foo"
// myProp("bar") will set the value to "bar" and return "bar"
// myProp.code() will return something like '.myProp("bar")' (with more whitespace)
// When you want to define a new, more specific kind of property, you'll generally need
// to override one or more of: my.set, my.formValue, my.genForm
// See pw.nameProperty and pw.typeProperty for examples of this
// You may also need to override the public method #code in your
// descendant's constructor.
pw.property = function(spec, my) {
    // *** Private data & methods
	var visible = true,
		accessor = function(newVal) {
			if (typeof newVal !== 'undefined') {
				my.set(newVal);
				// re-render the visualization each time a property's value changes
				pw.render();
			}
			return my.value;
		};

    // *** Data and methods shared with descendants during construction
    my = my || {};
    my.name = spec.name;
    // Un-camelcase the property name to display a friendlier label
    my.prettyName = my.name.replace(/([A-Z])/, function(cap) {return " " + cap.toLowerCase();});
    my.mark = spec.mark;
    my.value = spec.value || '';
    my.form = null;
    my.accessor = accessor;

    // my.set MUST do all of the following:
    // 1) Do any necessary validation on the new property value
    // 2) Change the state of the property (at minimum, set my.value = newVal)
    //    if the newVal passes validation (or can be munged into passing)
    // 3) Update the UI (i.e. my.form) to reflect the new/munged value
    my.set = my.set || function(newVal) {
        my.value = newVal;
        $('input', my.form).val(my.value);
    };
    // my.formValue returns an appropriate underlying value
    // based on the current state of my.form
    my.formValue = my.formValue || function() {
        return $('input', my.form).val();
    }
    // my.genForm is responsible for creating the guts of the property's form
    // it is only used at construction time
    my.genForm = my.genForm || function() {
        var input = $('<input/>', {
            value: my.value
        });
        return $('<label/>').append(document.createTextNode(my.prettyName + ' '), input);
    };
    
    my.code = my.code || function() {
        if (my.value) {
            return '\n    .' + my.name + '(' + pw.expand(my.value) + ')';
        } else {
            return '';
        }
    }

    // *** Public methods
    accessor.getName = function() {
        return my.name;
    };
       
    accessor.code = function() {
        return visible ? my.code() : '';
    };

    // set the visibility of this property
    // properties not used by the current mark type are made invisible
    // note that currently invisible (i.e. unused) properties return an empty code string,
    // as they are not relevant to the current mark type
    accessor.setVisible = function(v) {
        v = v || false;
        // If this is an advanced property, and we're not showing advanced properties
        // right now, override any attempt to set the visibility to true
        if (pw.showAdvancedProperties === false && pw.advancedProperties[my.name]) {
            v = false;
        }
        visible = v;
        my.form.toggle(v).toggleClass('visible', v);
    };

    // *** Initialization -- create the form and append it to the containing mark's form
    // generate the property's form and enclose it in a standard container
    my.form = $('<p/>')
        .addClass('property')
        .addClass(my.name + 'Property')
        .append(my.genForm());
    // watch all the form elements in the container and update the property when they change
    $('input, select, textarea', my.form).change(function() {
        // trim whitespace from all input elements before trying to use them
        $('input, textarea', my.form).val(function(i, value) {
           return $.trim(value); 
        });
        my.accessor(my.formValue());
    });
    // attach the property's form to its mark's form
    my.mark.appendToForm(my.form);

    return accessor;
};

pw.nameProperty = function(spec) {
    var my = {
        // nameProperty pretty much has a standard form, except that it has no surrounding label
        genForm: function() {
            var input = $('<input/>', {
                value: my.value,
				placeholder: "Enter a name",
				"class": "nameProperty"
            });
            return input;
        },
        // nameProperty makes sure the desired name is different than the current name,
        // and is a valid, unique identifier before using it
        set: function(newName) {
            if (newName === my.value) {
                return;
            }
            newName = pw.uniqueName(newName, my.mark.basicType + pw.id());
            my.value = newName;
            $('input', this.form).val(newName);
            // Make sure that any lists of marks are aware of this name change
            my.mark.updateLists();
        }
    };
    spec.value = pw.uniqueName(spec.value, spec.mark.basicType + pw.id());
    return pw.property(spec, my);
}

pw.typeProperty = function(spec) {
    var my = {
        formValue: function() {
            return $('select', my.form).val(); 
        },
        genForm: function() {
            // build type selector
            var select = $('<select/>');
			select.attr('name', 'typeProperty');
            var selected = false;

            $.each(spec.supportedTypes, function(i, mtype) {
                // don't let users create generic Marks or Scales, only "concrete" types of marks
                if (mtype != my.mark.basicType) {
                    selected = mtype === my.value ? ' selected="selected"' : '';
                    select.append('<option value="' + mtype + '"' + selected + '>' + mtype + '</option>');
                }
            });
            return $('<label for="typeProperty">type: </label>').append(select);
        },
        set: function(newType) {
            my.value = newType;
            $('select', my.form).val(my.value);
            // typeProperty ensures that all properties have their visibility set properly
            // after the mark's type is changed
            my.mark.updateVisibility();
            if (my.mark.properties.parent) {
                my.mark.properties.parent.updateForm();
            }
            // Add/remove ticks as data options if we just changed a scale
            // from/to ordinal, or if we just changed this mark's type
            // (only Rules get ticks as data options)
            if (my.mark.basicType === 'Scale') {
                $.each(pw.marks.filter(function(m) {return m.properties.type() === 'Rule'}), function(i, mark) {
                   mark.properties.data.updateForm(); 
                });
            } else { // this is a Mark, not a Scale
                my.mark.properties.data.updateForm();
            }
        }
    };
    return pw.property(spec, my);
}

// What kind of scale is this, spatial or color?
pw.kindProperty = function(spec) {
    var my = {
        formValue: function() {
            return $('select', my.form).val(); 
        },
        genForm: function() {
            // build type selector
            var select = $('<select/>');
			select.attr('name', 'typeProperty');
            var selected = false;

            $.each(["spatial", "color"], function(i, kind) {
                selected = kind === my.value ? ' selected="selected"' : '';
                select.append('<option value="' + kind + '"' + selected + '>' + kind + '</option>');
            });
            return $('<label>spatial/color: </label>').append(select);
        },
        set: function(newKind) {
            my.value = newKind;
            $('select', my.form).val(my.value);

            $.each(["range", "ordinalRange"], function(i, prop) {
                if (my.mark.properties[prop]) {
                    my.mark.properties[prop].updateForm();
                }
            });
        },
        // kindProperty never has any code, it's just for controlling the UI of the range properties
        code: function() {return '';}
    };
    spec.value = spec.value || 'spatial';
    return pw.property(spec, my);
}


// Property that keeps track of a mark's parent in the inheritance chain
pw.parentProperty = function(spec) {
    var noneOption = '<option value="">[none]</option>',
        my = {
            formValue: function() {
                return {
                    parent: $('.parentSelect', my.form).val(),
                    anchor: $('.anchorSelect', my.form).val(),
                };
            },
            genForm: function() {
                // build empty parent selector
                // #updateForm will take care of filling this in with mark names
                var pSelect = $('<select/>', {'class': 'parentSelect'}),
                    f = $('<label>based on: </label>').append(pSelect),
                    aSelect = pw.anchorSelect(),
                    anchorLabel = $('<label class="anchorLabel"> anchor: </label>')
                        .toggle(parseInt(my.value.parent) >= 0)
                        .append(aSelect);
       			
				// If the parent select changes, reflect that in the UI's header
                pSelect.bind('change', function() {
					pw.layersUI.writeInheritanceToHeader();
				});
				
                // Only Panels can inherit from nothing
                if (my.mark.properties.type() === 'Panel') {
                    pSelect.append(noneOption);
                }
                
                f = f.append(anchorLabel);
                return f;
            },
            set: function(newParent) {
                if (typeof newParent === 'object') {
                    my.value.parent = newParent.parent;
                    my.value.anchor = newParent.anchor;
                } else if (parseInt(newParent) >= 0){
                    my.value.parent = parseInt(newParent);
                } else {
                    // we didn't get a value we can use, so select no parent
                    my.value.parent = '';
                }
                $('.parentSelect', my.form).val(my.value.parent);
                $('.anchorSelect', my.form).val(my.value.anchor);
                // now toggle the visibility of the anchor part of the form depending on whether there's a parent
                var showAnchor = my.value.parent >= 0;
                $('.anchorLabel', my.form).toggle(showAnchor);
                
                my.mark.updateScaleAndFieldProperties();
            },
            code: function() {
                if (my.value.parent === '') {
                    return '';
                } else {
                    var c = pw.marks[my.value.parent].properties.name();
                    if (my.value.anchor) {
                        c += '.anchor(' + pw.expand(my.value.anchor) + ')';
                    }
                    return c;
                }
            }
        };
        
    // if the spec didn't pass in a value, we'll set up an empty
    // two-part value.  If a single value (not an object) was passed in
    // we'll assume that's referring to the parent layer, not the anchor
    if (typeof spec.value === 'undefined') {
        spec.value = {parent: pw.marks[0] ? 0 : '', anchor: ''};
    } else if (typeof spec.value !== 'object') {
        spec.value = {parent: parseInt(spec.value), anchor: ''}
    } // else we're assuming a usable object was passed in as spec.value
        
    var prop = pw.property(spec, my);
    
    // Update the list of possible parents
    // Called whenever the marks list changes
    prop.updateForm = function() {
        var select = $('.parentSelect', my.form).empty(),
            selected = false,
            mName = null,
            foundCurrentParent = false;
        
        // Only Panels can inherit from nothing
        if (my.mark.properties.type() === 'Panel') {
            select.append(noneOption);
        }
        
        // Build a new option list of possible parents (marks in a lower layer)
        for (var i=0; i<my.mark.layer; i++) {
            mName = pw.marks[i].properties.name();
            mLayer = pw.marks[i].layer;
            // If the current option corresponds to this property's current value,
            // select it and note that we found the existing parent on the list
            if (mLayer.toString() === my.value.parent.toString()) {
                foundCurrentParent = true;
                selected = ' selected="selected"';
            } else {
                selected = '';
            }           
            select.append('<option value="' + mLayer + '"' + selected + '>' + mName + '</option>');
        }
        
        // If we never found the current parent (e.g., because it was just deleted)
        // then we need to reset the property's value and make sure the "none"
        // option is selected
        if (!foundCurrentParent) {
            my.accessor('');
        }
    }
   
    return prop;
} // 

pw.tickLabelProperty = function(spec) {
    var my = {
        formValue: function() {
            return $('.anchorSelect', my.form).val();
        },
        genForm: function() {
            var aSelect = pw.anchorSelect(),
                f = $('<label class="anchorLabel">' + my.prettyName + ' </label>')
                .append(aSelect);
            pw.setSimpleCombo(aSelect, my.value);
            return f;
        },
        set: function(newVal) {
            my.value = newVal;
            pw.setSimpleCombo($('.anchorSelect', my.form), newVal);
        },
        code: function() {
            if (my.value === '') {
                return '';
            } else {
                var d = my.mark.properties.data(),
                    textProp;
                if (d.substring(0, 5) === 'scale') {
                    textProp = '.text(' + pw.scales[d.substring(5)].properties.name() + '.tickFormat)';
                } else {
                    textProp = '';
                }
                return "\n    .anchor(" + my.value + ").add(pv.Label)" + textProp;
            }
        }        
    };
    
    return pw.property(spec, my);
}

// Property that keeps track of a mark's (or scale's) dataSet
pw.dataProperty = function(spec) {
    var noneOption = function() {
            var txt = '[none]';
            if (hasParent() && my.mark.properties.parent().parent !== '') {
                txt = 'inherited from parent';
            }
            return '<option value="">' + txt + '</option>';
        },
        hasParent = function() {
            return my.mark.properties.parent ? true : false;
        },
        my = {
            formValue: function() {
                return $('select', my.form).val();
            },
            genForm: function() {
                // build empty dataSet selector
                // #updateForm will take care of filling this in with dataSet names
                var select = $('<select/>').append(noneOption());
                return $('<label>' + my.prettyName + ': </label>').append(select);
            },
            set: function(newVal) {
                my.value = newVal;
                $('select', this.form).val(newVal);
                my.mark.updateScaleAndFieldProperties({data: true});
            },
            code: function() {
                if (my.value === '') {
                    return '';
                } else {
                    var val;
                    if (pw.dataSets[my.value]) {
                        val = pw.dataSets[my.value].properties.name();
                    } else if (/^scale/.test(my.value)) {
                        var i = my.value.substring(5);
                        val = pw.scales[i].properties.name() + '.ticks()';
                    } else { // Data is something else, e.g. "d.values" when nesting
                        val = pw.expand(my.value);
                    }
                    return "\n    ." + my.name + "(" + val + ")";
                }
            }
        };
        
    var prop = pw.property(spec, my);
    
    // Update the list of possible dataSets
    // Called whenever the dataSets list changes
    prop.updateForm = function() {
        var select = $('select', my.form).empty().append(noneOption()),
            parent_ds = null,
            foundSelected = false;

        if (my.mark.properties.parent && pw.marks[my.mark.properties.parent().parent]) {
            parent_ds = pw.marks[my.mark.properties.parent().parent].properties.data.dataSet();
        }

        if (parent_ds && parent_ds.properties.nest()) {
            var selected = '';
            if (my.value === 'd.values') {
                selected = 'selected="selected"';
                foundSelected = true;
            }
            select.append('<option value="d.values" ' + selected + '>' + parent_ds.properties.name() + ' nested values</option>');
        }
        
        foundSelected = pw.dataSets.options(select, my.value) || foundSelected;

        // A rule can take a scale's ticks as data
        // (Well, so can any mark, but exposing it anywhere except Rule
        // is probably more confusing than helpful)    
        if (my.mark.properties.type() === 'Rule') {
            var valueFunc = function(i, scale) {
                if (scale.properties.type() === 'ordinal') {
                    return '';
                } else {
                    return 'scale' + i;
                }
            },
            textFunc = function(i, scale) {
                if (scale.properties.type() === 'ordinal') {
                    return '(' + scale.properties.name() + ' has no ticks)';
                } else {
                    return scale.properties.name() + ' ticks';
                }
            };        
        
            foundSelected = pw.appendOptions(select, pw.scales, my.value, valueFunc, textFunc) || foundSelected;
        }

        // If we never found the current dataSet (e.g., because it was just deleted)
        // then we need to reset the property's value and make sure the "none"
        // option is selected
        if (!foundSelected) {
            my.accessor('');
        }
    }
    
    // Convenience method; returns the dataSet object currently pointed to by this property
    prop.dataSet = function() {
        if (hasParent() && (my.value === '' || my.value === 'd.values')) {
            var p = my.mark.properties.parent().parent;
            if (p === '') {
                return null;
            } else {
                return pw.marks[p].properties.data.dataSet();
            }
        } else {
            return pw.dataSets[my.value];
        }
    }
    
    prop.scaleProperty = true;
    prop.updateForm();
    
    return prop;
}

// This is the basic factory for a property that often applies a scale to a value
// I.e., posProperty, styleProperty
pw.scaleAndFieldProperty = function(spec, my) {
    var noneOption = '<option></option>';
    my = my || {};
    
    my.basicFormValue = function() {
        var scale = $('.scaleSelect', my.form).val(),
            field = $('.fieldSelect', my.form).val();
        return {field: field, scale: scale};
    };
    
    my.formValue = my.formValue || my.basicFormValue;
    
    my.basicGenForm = function(initVal) {
        initVal = initVal || '';
        // For some reason f.after() won't work if I start f with a <label>,
        // so I'll use a dummy <div> that I'll chop off at the end.
        var f = $('<div/>'),
            scaleSelect = $('<select/>', {'class': 'scaleSelect'}).append(noneOption),
            fieldSelect = $('<select/>', {'class': 'fieldSelect'}).append(noneOption),
            ds = my.mark.properties.data.dataSet();

        if (ds) {
            ds.options(fieldSelect);   
        }
        fieldSelect.simpleCombo();
        pw.setSimpleCombo(fieldSelect, my.value.field);

        f = f.after($('<label>: use data/value </label>').append(fieldSelect));

        var res = pw.scales.options(scaleSelect, my.value.scale);
        f = f.after('<br/>').after($('<label> apply scale </label>').append(scaleSelect));

        return f.slice(1);
    }
    
    // posProperty and styleProperty will provide their own genForm
    // so they can add some extra selects
    // Pretty much all other scaleAndFieldProperties just need to tack
    // their name onto the beginning of the form
    my.genForm = my.genForm || function() {
        // Again, we'll use the dummy <div> trick since .after doesn't
        // work if you start with a <label>
        var f = $('<div/>');
        f = f.after($('<label>' + my.prettyName + '</label>'));
        f = f.after(my.basicGenForm(my.value));
        return f.slice(1);
    }
    
    my.basicSet = function(newVal) {
        my.value.field = newVal.field;
        my.value.scale = newVal.scale;
        
        pw.setSimpleCombo($('.fieldSelect', my.form), my.value.field);        
        $('.scaleSelect', my.form).val(my.value.scale);
    }
    
    my.set = my.set || my.basicSet;
    
    my.code = my.code || function() {
        var field = my.value.field,
            scale = my.value.scale,
            val = '',
            // if this is actually a posProperty, it will have a "side" value
            // (like 'left') and we'll need to use that for the property name
            pName = my.value.side ? my.value.side : my.name;

        if (field !== '' && scale !== '') {
            val = pw.scales[scale].properties.name() + '(' + field + ')';
        } else {
            val = pw.scales[scale] ? pw.scales[scale].properties.name() : field;
        }
        
        if (val !== '') {
            return '\n    .' + pName + '(' + pw.expand(val) + ')';
        } else {
            return '';
        }
    }
    
    spec.value = spec.value || {field: '', scale: ''};
    
    var prop = pw.property(spec, my);
    
    // the form will need to be updated any time the list of scales changes,
    // or the associated mark's data or parent property changes
    prop.updateForm = function() {
        var scaleSelect = $('.scaleSelect', my.form),
            fieldSelect = $('.fieldSelect', my.form);
        
        scaleVal = scaleSelect.val();
        fieldVal = fieldSelect.val();
        
        scaleSelect = scaleSelect.empty().append(noneOption);
        fieldSelect.children("option:not(:first)").remove();
        
        var foundScale = pw.scales.options(scaleSelect, scaleVal);
        if (!foundScale) {
            my.value.scale = '';
        }
        
        var ds = my.mark.properties.data.dataSet(),
            foundField = false;
        if (ds) {
            foundField = ds.options(fieldSelect, fieldVal);   
        }
        // If the value in the fieldSelect looks like "d.*" and it wasn't just found in
        // the list of available data fields (i.e. because this mark's dataSet has changed)
        // then unset the field and scale.
        if (/^d\./.test(fieldVal) && !foundField) {
            my.basicSet({field: '', scale: ''});
        }     
    }
    
    prop.fieldProperty = true;
    prop.scaleProperty = true;
    
    return prop;
}

pw.angleProperty = function(spec) {
    var my = {};
    return pw.scaleAndFieldProperty(spec, my);
}
pw.heightProperty = pw.angleProperty;
pw.widthProperty = pw.angleProperty;
pw.textProperty = pw.angleProperty;

pw.posProperty = function(spec) {
    var sideOptions = null;
    switch (spec.dir) {
        case 's': sideOptions = ['size', 'radius']; break;
        case 'y': sideOptions = ['bottom', 'top']; break;
        default: sideOptions = ['left', 'right'];
    }
    
    var my = {
            formValue: function() {
                var val = my.basicFormValue();
                val.side = $('.sideSelect', my.form).val();
                return val;
            },
            genForm: function() {
                var f = $('<select/>', {'class': 'sideSelect'});
                $.each(sideOptions, function(i, side) {
                    f.append('<option value="' + side + '">' + side + '</option>');
                });
                if (my.value.side) {
                    f.val(my.value.side);
                }
                return f.after(my.basicGenForm(my.value.pos));;
            },
            set: function(newPos) {
                my.value.side = newPos.side;
                $('.sideSelect', my.form).val(my.value.side);
                my.basicSet(newPos);
            },
        };
        
    // if the spec didn't pass in a value, we'll set up an empty
    // three-part value.
    if (typeof spec.value === 'undefined') {
        spec.value = {side: sideOptions[0], field: '', scale: ''};
    } // else we're assuming a usable object was passed in as spec.value
        
    var prop = pw.scaleAndFieldProperty(spec, my);

    return prop;    
}

pw.xProperty = function(spec) {
    spec.dir = 'x';
    return pw.posProperty(spec);
}

pw.yProperty = function(spec) {
    spec.dir = 'y';
    return pw.posProperty(spec);
}

pw.sizeProperty = function(spec) {
    spec.dir = 's';
    return pw.posProperty(spec); 
}

pw.styleProperty = function(spec) {   
    var noneOption = '<option></option>',
        my = {
            formValue: function() {
                var val = my.basicFormValue();
                val.alpha = $('.alphaSelect', my.form).val();
                return val;
            },
            genForm: function() {
                // Again, we'll use the dummy <div> trick since .after doesn't
                // work if you start with a <label>
                var f = $('<div/>');
                f = f.after($('<label>' + my.prettyName + '</label>'));
                f = f.after(my.basicGenForm(my.value));
                
                var alphas = pv.range(0, 1, 0.1);
                    aSelect = $('<select/>').addClass('alphaSelect').append(noneOption),
                    valueFunc = function(i, v) {return v};
                
                for (var i=0; i<alphas.length; i++) {
                    alphas[i] = alphas[i].toFixed(1);
                }
                
                pw.appendOptions(aSelect, alphas, my.value.alpha, valueFunc);                
                aSelect.simpleCombo();
                pw.attachPicker($('.fieldSelect', f));
                f = f.after('<br/>').after($('<label> alpha </label>').append(aSelect));
                
                return f.slice(1);
            },
            set: function(newVal) {
                my.value.alpha = newVal.alpha;
                $('.alphaSelect', my.form).val(my.value.alpha);
                // If the field has a color picker value in it, unset the scale
                // (we don't want to scale a color literal)
                if (/^['"]#/.test(newVal.field)) {
                    newVal.scale = '';
                }
                my.basicSet(newVal);
            },
            code: function() {
                var field = my.value.field,
                    scale = my.value.scale,
                    alpha = my.value.alpha,
                    val = '';

                if (field !== '') {
                    if (scale !== '') {
                        val = pw.scales[scale].properties.name() + '(' + field + ')';
                        if (alpha !== '') {
                            val += '.alpha(' + alpha + ')';
                        }
                    } else {
                        if (alpha === '') {
                            val = field;
                        } else {
                            val = 'pv.color(' + field + ').alpha(' + alpha + ')';
                        }
                    }
                }

                if (val !== '') {
                    return '\n    .' + my.name + '(' + pw.expand(val) + ')';
                } else {
                    return '';
                }
            }
        };
        
    // if the spec didn't pass in a value, we'll set up an empty
    // three-part value.
    if (typeof spec.value === 'undefined') {
        spec.value = {field: '', scale: '', alpha: ''};
    } // else we're assuming a usable object was passed in as spec.value
    
    if (typeof spec.value.alpha === 'undefined') {
        spec.value.alpha = '';
    }
        
    var prop = pw.scaleAndFieldProperty(spec, my);

    return prop;
}
pw.strokeStyleProperty = pw.styleProperty;
pw.fillStyleProperty  = pw.styleProperty;
pw.textStyleProperty  = pw.styleProperty;

// A generic property where values are chosen from a list
pw.listProperty = function(spec) {
    var my = {
        formValue: function() {
            return $('select', my.form).val();
        },
        // a listProperty has a standard form, except with a select box
        genForm: function() {
            var select = $('<select/>', {'class': 'listPropertySelect'});
            // Assume the first option in the list is the default
            // This means that it should have a blank option value, so that we don't
            // write out protovis code just to affirm the default
            select.append('<option value="">' + spec.listOptions.shift() + '</option>');
            $.each(spec.listOptions, function(i, v) {
               select.append('<option value="' + v + '">' + v + '</option>'); 
            });
            return $('<label/>').append(document.createTextNode(my.prettyName + ' '), select);
        },
        set: function(newVal) {
            my.value = newVal;
            $('select', this.form).val(newVal);
        },
        code: function() {
            var c = '';
            if (my.value !== '') {
                c += '\n    .' + my.name + '("' + my.value + '")';
            }
            return c;
        }
    };
    var prop = pw.property(spec, my);

    return prop;
}

pw.cursorProperty = function(spec) {
    spec.listOptions = [
        "",
        "inherit",
        "default",
        "auto",
        "n-resize",
        "ne-resize",
        "e-resize",
        "se-resize",
        "s-resize",
        "sw-resize",
        "w-resize",
        "nw-resize",
        "crosshair",
        "pointer",
        "move",
        "text",
        "wait",
        "help",
        "hand",
        "all-scroll",
        "col-resize",
        "row-resize",
        "no-drop",
        "not-allowed",
        "progress",
        "vertical-text",
        "alias",
        "cell",
        "copy",
        "count-down",
        "count-up",
        "count-up-down",
        "grab",
        "grabbing",
        "spinning"
    ];
    return pw.listProperty(spec);
}

pw.shapeProperty = function(spec) {
    spec.listOptions = [
        "circle",
        "cross",
        "triangle",
        "diamond",
        "square",
        "tick",
        "bar"
    ];
    return pw.listProperty(spec);
}

pw.interpolateProperty = function(spec) {
    spec.listOptions = [
        "linear",
        "step-before",
        "step-after",
        "polar",
        "polar-reverse",
        "basis",
        "cardinal"
    ];
    return pw.listProperty(spec);
}

pw.lineJoinProperty = function(spec) {
    spec.listOptions = [
        "miter",
        "bevel",
        "round"
    ];
    return pw.listProperty(spec);
}

pw.textAlignProperty = function(spec) {
    spec.listOptions = [
        "left",
        "center",
        "right"
    ];
    return pw.listProperty(spec);
}

pw.textAlignProperty = function(spec) {
    spec.listOptions = [
        "bottom",
        "middle",
        "top"
    ];
    return pw.listProperty(spec);
}

// Note: technically the text decoration can also have a style (e.g. dotted)
// and a color.  I'm not worrying about supporting that yet since the need 
// is quite rare.
pw.textDecorationProperty = function(spec) {
    spec.listOptions = [
        "none",
        "underline",
        "line-through",
        "overline"
    ];
    return pw.listProperty(spec);
}

// A property whose value is generally either true or false
// spec.def should tell us whether protovis defaults the property to true or false
// If spec.def is undefined, booleanProperty will treat the property as
// defaulting to false
pw.booleanProperty = function(spec, def) {
    var my = {
        formValue: function() {
            return $('input', my.form).is(':checked');
        },
        // a booleanProperty has a standard form, except with a checkbox
        genForm: function() {
            var checked = def ? 'checked' : '',
                input = $('<input/>', {
                    type: 'checkbox',
                    'class': 'propertyCheckbox',
                    checked: checked
                });
            return $('<label/>').append(document.createTextNode(my.prettyName), input);
        },
        set: function(newVal) {
            my.value = newVal;
            var checked = newVal ? 'checked' : '';
            $('input', my.form).attr('checked', checked);
        },
        code: function() {
            var c = '';
            if (my.value != def) {
                c += '\n    .' + my.name + '(' + my.value.toString() + ')';
            }
            return c;
        }
    };
    spec.value = def;
    var prop = pw.property(spec, my);   

    // By default, pw.property sets my.value to '' (empty string)
    // but booleanProperty is expecting its value to be an actual boolean,
    // so we'll fudge that here.
    if (!my.value) my.value = false;

    return prop;
}

pw.visibleProperty = function(spec) {
    return pw.booleanProperty(spec, true);
}

pw.reverseProperty = function(spec) {
    return pw.booleanProperty(spec, false);
}
pw.antialiasProperty = pw.reverseProperty;
pw.segmentedProperty = pw.reverseProperty;

pw.dataSetProperty = function(spec) {
    var revert = function() {
            $('textarea', this.form).val(my.value);
        },
        // Take javascript code reperesenting data, and try to parse it to an object
        // If the parse is successful, and the object is an array, return it
        // Otherwise, return null
        textToArray = function(dataText) {
            try {
                var dataArr = eval(dataText);
            } catch(e) {
                alert("I couldn't parse that data set; reverting to the old/default data.");
                return null;
            }
            
            if (dataArr.constructor != Array) {
                alert("I successfully parsed your data, but it doesn't seem to be an array. Reverting to the old/default data.")
                return null;
            }
            
            return dataArr;
        },
        fieldMap = {},
        fieldList = [],
        // Update the field list derived from this data.
        // For now, we're just being very simple and assuming that the fields
        // are all named in the first object in the data array
        updateFieldList = function(dataArr) {
            fieldMap = {};
            fieldList = [];
            // We can assume at this point that dataArr is an array,
            // but it could be an empty array, or an array of basic data
            // (like numbers) in which case we'll leave the field list blank.
            if (dataArr[0] && typeof dataArr[0] === 'object') {
                $.each(dataArr[0], function(i, v) {
                    fieldMap[i] = true;
                    fieldList.push(i);
                });
            }
            fieldList.sort(); 
        },
        my = {
            formValue: function() {
                return $('textarea', my.form).val();
            },
            // dataSetProperty pretty much has a standard form, except that it has a textarea for pasting into
            genForm: function() {
                return $('<textarea/>', {
                    value: my.value
                });
            },
            set: function(newData) {
                if (newData === my.value) {
                    return;
                }
            
                var dataArr = textToArray(newData);
                if (dataArr === null) {
                    revert();
                    return;
                }
                updateFieldList(dataArr);
            
                my.value = newData;
            
                $('textarea', this.form).val(newData);
            },
            code: function() {
                return my.value;
            }
        };

    var that = pw.property(spec, my);
    
    // Returns the field list as an unordered map
    // or an ordered array, depending on the parameter
    that.fields = function(ordered) {
        return ordered ? fieldList : fieldMap;
    }
    
    var dataArr = textToArray(my.value);
    // if we got initialized with bad data, throw it out
    if (dataArr === null) {
        dataArr = [];
        my.value = '[]';
        $('textarea', my.form).val(my.value);
    }
    updateFieldList(dataArr);
    delete dataArr; // no reason to keep dataArr in memory now; it might be big.
    
    return that;
}

pw.nestProperty = function(spec) {
    var noneOption = '<option></option>',
        my = {
            formValue: function() {
                return $('.fieldSelect', my.form).val();
            },
            genForm: function(initVal) {
                initVal = initVal || '';
                var label = my.name === 'nest' ? 'nest by: ' : 'then by: ';
                    f = $('<label/>').text(label),
                    fieldSelect = $('<select/>', {'class': 'fieldSelect'}).append(noneOption);
                
                // We'll need to let updateForm take care of adding the data field options,
                // since at this point this property's dataSet doesn't have the #fields
                // or #options methods defined yet
                
                fieldSelect.simpleCombo();
                pw.setSimpleCombo(fieldSelect, my.value.field);

                return f.append(fieldSelect);
            },
            set: function(newVal) {
                my.value = newVal;
                pw.setSimpleCombo($('.fieldSelect', my.form), my.value);
                $.each(pw.marks, function(i, mark) {
                    mark.properties.data.updateForm();
                });
            },
            code: function() {
                if (my.value === '') {
                    return '';
                } else {
                    return '\n    .key(' + pw.expand(my.value) + ')';
                }
            }
        }
    
    var prop = pw.property(spec, my);

    prop.updateForm = function() {
        var fieldSelect = $('.fieldSelect', my.form),
            fieldVal = fieldSelect.val();
            
        fieldSelect.children("option:not(:first)").remove();
        
        var ds = my.mark,
            foundField = false;
        if (ds) {
            foundField = ds.options(fieldSelect, fieldVal);   
        }
        // If the value in the fieldSelect looks like "d.*" and it wasn't just found in
        // the list of available data fields (i.e. because this scale's dataSet has changed)
        // then unset the field and scale.
        if (/^d\./.test(fieldVal) && !foundField) {
            my.basicSet({field: '', scale: ''});
        }
    }
    
    prop.fieldProperty = true;
    
    return prop;
}

pw.nest2Property = pw.nestProperty;

pw.domainRangeProperty = function(spec, my) {
    var noneOption = '<option></option>'; // simpleCombo gets weird if you try to put 'value=""' in there
    
    my.formValue = function() {
        var from = $('.from', my.form).val();
        var to = $('.to', my.form).val();
        return { from: from, to: to };
    }
    my.genForm = function() {
        var f = $('<label>' + my.prettyName + ' <select class="from"/></label><label>...<select class="to"/></label>');
        $('select', f).append(noneOption).simpleCombo();
        
        // Make the first <option> in each select list show its corresponding initial value
        $.each(my.value, function(i, v) {
            pw.setSimpleCombo($('.' + i, f), v);
        });

        return f;
    }
    my.code = function() {
        if (my.value.from === '' || my.value.to === '') {
            return '';
        } else {
            return my.value.from + ', ' + my.value.to;
        }
    }
 
    // Set up a value based on a passed-in string
    if (typeof spec.value === 'string') {
        var split = spec.value.split(/\s*,\s*/);
        spec.value = {
            from: split[0],
            to: split[1]
        }
    // If we didn't get a string, see if we got an object
    // If we got an object, we'll assume it's got the right properties and use it
    // Otherwise, we'll set up an empty value object
    } else if (typeof spec.value !== 'object') {
        spec.value = {from: '', to: ''};
    }
    
    var prop = pw.property(spec, my);
    
    return prop;
}

pw.domainProperty = function(spec) {
    var my = {
            set: function(newDomain) {
                my.value = newDomain;
                $('.from', my.form).val(my.value.from);
                $('.to', my.form).val(my.value.to);
            }
        },
        prop = pw.domainRangeProperty(spec, my);

    prop.updateForm = function() {
        var from = $('.from', my.form),
            to = $('.to', my.form),
            ds = my.mark.properties.data.dataSet(),
            fromValFunc = function(i, v) {return "pv.min(" + ds.properties.name() + ", function(d) {return d." + v + "})"},
            fromTextFunc = function(i, v) {return 'min of: ' + v },
            toValFunc = function(i, v) {return "pv.max(" + ds.properties.name() + ", function(d) {return d." + v + "})"},
            toTextFunc = function(i, v) {return 'max of: ' + v };

        $.each([from, to], function(i, select) {
            select.children("option:not(:first)").remove();
        });        
        if (ds) {
            pw.appendOptions(from, ds.fields(true), my.value.from, fromValFunc, fromTextFunc);
            pw.appendOptions(to, ds.fields(true), my.value.to, toValFunc, toTextFunc);
        }
    }
    
    prop.fieldProperty = true;
    prop.updateForm();   
      
    return prop;
}

pw.rangeProperty = function(spec) {
    var jscolors = [],
        my = {
            set: function(newVal) {
                my.value = newVal;
                $('.from', my.form).val(my.value.from);
                if (!(/^["'].+["']$/.test(my.value.from))) {
                    $('.from+.picker', my.form).css('background-color', 'none');
                }
                $('.to', my.form).val(my.value.to);
                if (!(/^["'].+["']$/.test(my.value.to))) {
                    $('.to+.picker', my.form).css('background-color', 'none');
                }
            }
        },
        prop = pw.domainRangeProperty(spec, my),
        appendAngleOptions = function(select) {
            select.append('<option value="0">0</option>')
                .append('<option value="Math.PI/4">45&deg; (&pi;/4)</option>')
                .append('<option value="Math.PI/2">90&deg; (&pi;/2)</option>')
                .append('<option value="3*Math.PI/4">135&deg; (3&pi;/4)</option>')
                .append('<option value="Math.PI">180&deg; (&pi;)</option>')
                .append('<option value="5*Math.PI/4">225&deg; (5&pi;/4)</option>')
                .append('<option value="3*Math.PI/2">270&deg; (3&pi;/2)</option>')
                .append('<option value="7*Math.PI/4">315&deg; (7&pi;/4)</option>')
                .append('<option value="2*Math.PI">360&deg; (2&pi;)</option>');
        },
        appendProtovisColors = function(select) {
            var names = ["blue", "orange", "green", "red", "purple", "brown", "pink", "gray", "yellow-green", "cyan"],
                vals = ["#1f77b4", "#ff7f0e", "#2ca02c", "#d62728", "#9467bd", "#8c564b", "#e377c2", "#7f7f7f", "#bcbd22", "#17becf"];
            
            for (var i=0; i<names.length; i++) {
                select.append('<option value="\'' + vals[i] + '\'">' + names[i] + '</option>');
            }
        }

    $.each(['from', 'to'], function(i, className) {
        jscolors.push(pw.attachPicker($('.' + className, my.form)));
    });
    
    prop.updateForm = function() {
        var newVal = $.extend({}, my.value);
        // Tricky.  It looks like you should be able to do $('select', my.form).children("option:not(:first)")
        // and avoid the .each, but that gets rid of *all* the options in the second select,
        // because only the first option in the *first* select (.from) counts as ":first"
        // in the combined list.
        $.each($('select', my.form), function(i, select) {
            $(select).children("option:not(:first)").remove();
        });
        if (my.mark.properties.kind() === 'spatial') { // spatial scale
            appendAngleOptions($('.from', my.form));
            
            var to = $('.to', my.form).append('<option value="w">canvas width (w)</option>')
                .append('<option value="h">canvas height (h)</option>');
            appendAngleOptions(to);

            $.each(['from', 'to'], function(i, className) {
                if (/^["'].+["']$/.test(my.value[className])) {
                    newVal[className] = '';
                    pw.setSimpleCombo($('.' + className, my.form), '');
                }
            });
            $('.picker', my.form).css('background-color', 'transparent').hide();
            $.each(jscolors, function(i, jsc) {
                jsc.enabled = false;
            });
        } else { // color scale
            appendProtovisColors($('.from', my.form));
            appendProtovisColors($('.to', my.form));
            $.each(['from', 'to'], function(i, className) {
                if (!(/^["'].+["']$/.test(my.value[className]))) {
                    newVal[className] = '';
                    pw.setSimpleCombo($('.' + className, my.form), '');
                }
            });
            $('.picker', my.form).show();
            $.each(jscolors, function(i, jsc) {
                jsc.enabled = true;
            });
        }
        prop(newVal)
    }
    prop.updateForm();
    
    return prop; 
}

pw.ordinalDomainProperty = function(spec) {
    var noneOption = '<option></option>',
        my = {
            formValue: function() {
                return $('.fieldSelect', my.form).val();
            },
            genForm: function(initVal) {
                initVal = initVal || '';
                var f = $('<label>domain: </label>'),
                    fieldSelect = $('<select/>', {'class': 'fieldSelect'}).append(noneOption),
                    ds = my.mark.properties.data.dataSet();

                if (ds) {
                    ds.options(fieldSelect);   
                }
                fieldSelect.simpleCombo();
                pw.setSimpleCombo(fieldSelect, my.value.field);

                return f.append(fieldSelect);
            },
            set: function(newVal) {
                my.value = newVal;
                pw.setSimpleCombo($('.fieldSelect', my.form), my.value);        
            },
            code: function() {
                var ds = my.mark.properties.data.dataSet();
                if (ds && /^d\./.test(my.value)) {
                    return ds.properties.name() + ', function(d) ' + my.value;
                } else {
                    return my.value;
                }
            }
        }
    
    var prop = pw.property(spec, my);

    prop.updateForm = function() {
        var fieldSelect = $('.fieldSelect', my.form),
            fieldVal = fieldSelect.val();
            
        fieldSelect.children("option:not(:first)").remove();
        
        var ds = my.mark.properties.data.dataSet(),
            foundField = false;
        if (ds) {
            foundField = ds.options(fieldSelect, fieldVal);   
        }
        // If the value in the fieldSelect looks like "d.*" and it wasn't just found in
        // the list of available data fields (i.e. because this scale's dataSet has changed)
        // then unset the field and scale.
        if (/^d\./.test(fieldVal) && !foundField) {
            my.basicSet({field: '', scale: ''});
        }
    }
    
    prop.fieldProperty = true;
    
    return prop;
}

pw.ordinalRangeProperty = function(spec) {
    var noneOption = '<option></option>',
        my = {
            formValue: function() {
                return $('.rangeSelect', my.form).val();
            },
            genForm: function(initVal) {
                initVal = initVal || '';
                var f = $('<label>range: </label>'),
                    rangeSelect = $('<select/>', {'class': 'rangeSelect'}).append(noneOption);

                $.each(['10', '19', '20'], function(i, v) {
                    rangeSelect.append('<option value="pv.Colors.category' + v + '">' + v + ' color</option>');
                });
                $.each(['w', 'h'], function(i, max) {
                    rangeSelect.append('<option value="split(0, ' + max + ')">split(0, ' + max + ')</option>');
                    rangeSelect.append('<option value="splitBanded(0, ' + max + ', 4/5)">splitBanded(0, ' + max + ', 4/5)</option>');
                    rangeSelect.append('<option value="splitFlush(0, ' + max + ')">splitFlush(0, ' + max + ')</option>');
                });

                rangeSelect.simpleCombo();
                pw.setSimpleCombo(rangeSelect, my.value.field);

                return f.append(rangeSelect);
            },
            set: function(newVal) {
                my.value = newVal;
                pw.setSimpleCombo($('.rangeSelect', my.form), my.value);        
            },
            code: function() {
                return my.value;
            }
        }

    prop = pw.property(spec, my);

    prop.updateForm = function() {
        var select = $('select', my.form),
            foundSelected = false;

        select.children("option:not(:first)").remove();

        if (my.mark.properties.kind() === 'spatial') { // spatial scale
            var bandOpts = ["split(0, h)", "splitBanded(0, h, 4/5)", "splitFlush(0, h)", "split(0, w)", "splitBanded(0, w, 4/5)", "splitFlush(0, w)"],
                textFunc = function(i, v) {return v};      
            foundSelected = pw.appendOptions(select, bandOpts, my.value, null, textFunc);               
        } else { // color scale
            var palettes = ['10', '19', '20'],
                valFunc = function(i, v) {return 'pv.Colors.category' + v},
                textFunc = function(i, v) {return v + ' color'};
            foundSelected = pw.appendOptions(select, palettes, my.value, valFunc, textFunc);
        }
        if (!foundSelected) {
            prop('');
        }
    }
    prop.updateForm();
    
    return prop;
}

// A protowiz Mark
// A Mark creates a form where all of its properties can be set
// It re-jiggers its form if its mark type changes
// The code() method returns the complete protovis code for the mark
pw.mark = function(spec, my) {
    // build this mark's input form
    var form = $('<div/>', {'class': 'mark'}),
		closer = $('<div class="closer">X</div>'),
        that = {},
        myDefaults = {
            allProperties: pw.allMarkProperties,
            list: pw.marks,
            container: $('#marks'),
            supportedTypes: pw.supportedMarks
        };

	// If there's no 'my' object passed in, create one.
    my = my || {};
	// Check to see whether the 'my' object has certain members, and if not write defaults to it.
    $.each(myDefaults, function(i, v) {
        my[i] = my[i] || v;
    });
    
    // basicType is the fundamental kind of protovis thing we're creating
    // e.g. 'Mark', 'Scale'
    that.basicType = spec.basicType || 'Mark';
    
    spec.name = spec.name || that.basicType;
	
	// Since we're not programmatically creating marks but based on user input, we need a way to store the form's code
	// in the returned object.
	that.formHTML = "";

    // Properties call this in order to attach their sub-form
    // to the parent mark's form
    that.appendToForm = function(node) {
        form.append(node);
    }

    // for each property: if it's used with this mark's basicType or its
    // specific type, make it visible. Otherwise, make it invisible.
    // E.g. if we're making a pv.Dot, properties for "Mark" & "Dot"
    // should be visible, but not properties for "Scale" or "Label"
    that.updateVisibility = function() {
        // descendants of mark that don't have a type (e.g. dataSet) need to short circuit this
        if (that.properties.type) {
            var type = that.properties.type();
            $.each(that.properties, function(i, prop) {
                prop.setVisible(pw.properties[that.basicType][i] || pw.properties[type][i]);
            });
        }
        if (that.basicType === 'Mark') {
            $('p.visible:even', form).css('background-color', 'transparent');
            $('p.visible:odd', form).css('background-color', '#bbb');
        }
    }

    that.code = function() {
        var parent = that.properties.parent.code();
            c = "\n\n" + that.properties.name() + " = ";
        
        if (parent === '') {
            c += "new pv." + that.properties.type().capitalize() + "().canvas('fig')";
        } else {
            c += parent + ".add(pv." + that.properties.type().capitalize() + ")";
        }

        // list of properties that shouldn't be part of the generic code generation loop
        var skip = {name: true, type: true, parent: true};

        for (p in that.properties) {
            if (!skip[p]) {
                c += that.properties[p].code();
            }
        }
        c += ';';
        return c;
    }

    that.del = function() {
        // jQuery commenters recommend using .empty() before .remove() for performance reasons
        form.empty().remove();
        my.list.del(that);
    }

	that.getParent = function() {
		return pw.marks.getObjectOnLayer(that.properties.parent().parent);
	};
	
	that.getChildren = function() {
		var children = [];
		for (var i=0; i < pw.marks.length; i++) {
			if (pw.marks[i].properties.parent().parent === that.layer) {
				children.push(pw.marks[i]);
			}; 
		};
		return children;
	};

    
    that.updateLists = function() {
        my.list.updateLists();
    }
    
    // skip map is needed to avoid infinite loops, e.g. when we are updating because
    // the data property changed, we don't want to try to update the data property's
    // form again.
    that.updateScaleAndFieldProperties = function(skipMap) {
        skipMap = skipMap || {};
        $.each(that.properties, function(name, prop) {
            if (prop && !skipMap[name] && (prop.scaleProperty || prop.fieldProperty)) {
                prop.updateForm();
            }
        });
    }
    
    // It seems like there should be a less awkward way of giving #del
    // the right context, but I can't think of one
    // closer.click(function() { that.del.call(that) });
    // form.append(closer);
        
    // create a new pw.property for each of this mark's properties
    // the property will take care of appending its form to the mark's form
    that.properties = {}
    var i, pName, pSpec;
    for(i=0; i<my.allProperties.length; i++) {
        pName = my.allProperties[i];
        pSpec = {name: pName, value: spec[pName], mark: that};
        if (pName === 'type') {
            pSpec.supportedTypes = my.supportedTypes;
        }
        // if the pw object has a special constructor for this kind of property
        // (e.g. pw["nameProperty"]) then use it to construct the property.
        // Otherwise, construct a generic pw.property.
        if (pw[pName + 'Property']) {
            that.properties[pName] = pw[pName + 'Property'](pSpec);
        } else {
            that.properties[pName] = pw.property(pSpec);
        }
    }

    // Being added to the marks list will set that.layer to this mark's
    // index in the marks array
    my.list.add(that);
  	my.container.append(form);
	that.formHTML = form;
	that.id = pw.id();
	that.updateVisibility()
    return that;
} // end pw.mark()

pw.scale = function(spec) {
    var my = {
        allProperties: pw.allScaleProperties,
        list: pw.scales,
        container: $('#scales'),
        supportedTypes: pw.supportedScales
    };
    spec.basicType = 'Scale';
	spec.type = spec.type || "linear";
    var that = pw.mark(spec, my);
    
    that.code = function() {
        var c = "\n" + that.properties.name() + " = ",
            domain,
            range;
        
        if (that.properties.type() === 'ordinal') {
            domain = that.properties.ordinalDomain.code();
            range = that.properties.ordinalRange.code();
        } else {
            domain = that.properties.domain.code();
            range = that.properties.range.code();
        } 
        
        // If the range is a standard protovis color scale, set up the scale code idiomatically
        if (/^pv\.Colors/.test(range)) {
            c += range + '(' + domain + ')';
        } else { // The range is not a standard color protovis color scale
            c += "pv.Scale." + that.properties.type() + "(" + domain + ")";        
            // If the range is an ordinal split*, use the .split*() method rather than .range() 
            if (/^split/.test(range)) {
                c += '.' + range;
            } else if (range !== '') {
                c += ".range(" + range + ")";
            }
        }
        return c + ';';
    }
    
    return that;
}

pw.dataSet = function(spec) {
    var my = {
        allProperties: pw.allDataSetProperties,
        list: pw.dataSets,
        container: $('#dataSets')
    };
    spec.basicType = 'dataSet';
    
    var that = {};
    
    var that = pw.mark(spec, my);
    
    that.code = function() {
        var c = "\n" + that.properties.name() + " = ";
        c += that.properties.dataSet();
        if (that.properties.nest() !== '') {
            c += "\n" + that.properties.name() + " = pv.nest(" + that.properties.name() + ")";
            c += that.properties.nest.code();  
            c += that.properties.nest2.code();
            c += '\n    .entries();'
        }
        return c;
    }
    
    // dataSets are always the same type (i.e., they don't have a "type" property)
    // so they never have to update their properties' visibility
    that.updateVisibility = function() {return;};
    
    // Return a field list from the dataSet property, ordered if the parameter evaluates to true
    // This is just a convenience method
    that.fields = function(ordered) {
        return that.properties.dataSet.fields(ordered);
    }
    
    // Append this dataSet's field names as a series of <options>
    // to the passed-in <select>
    that.options = function(select, selectedOpt) {
        var valueFunc = function(i, v) {return 'd.' + v};
        return pw.appendOptions(select, that.fields(true), selectedOpt, valueFunc, valueFunc);      
    }
    
    that.properties.nest.updateForm();
    that.properties.nest2.updateForm();
    
    return that;
}

pw.code = function() {
    var theCode = '\nvar w = ' + pw.w + '\n    h = ' + pw.h;
    
    $.each(pw.dataSets, function(i, dataSet) {
        theCode += dataSet.code();
    });

    $.each(pw.scales, function(i, scale) {
        theCode += scale.code();
    });
    
    $.each(pw.marks, function(i, mark) {
        theCode += mark.code();
    });
    
    theCode += "\n\n" + pw.rootName + ".render();";

    var showCode = theCode.replace(/\n/gm, '\n            ');
    $("#code").text(pw.layersUI.beforeCode + showCode + pw.layersUI.afterCode);
    
    return theCode;
}

pw.status = function(err) {
    var div = $('#status');
    if (err) {
        div.html(" &#10008; " + err).css('color', '#d62728');
    } else {
        div.html(" &#10004; OK").css('color', '#2ca02c');
    }
}
pw.status();

pw.render = function() {
    // pv.parse basically lets you use some common javascript 1.8 idioms.
    // I don't totally understand it, but it seems to be the special sauce
    // that makes this work.
    try {
        eval(pv.parse(pw.code()));
        pw.status();
    } catch(err) {
        pw.status(err);
    }
}

pw.updateHW = function() {
    var w = parseInt($('#width').val()),
        h = parseInt($('#height').val());

    if (isNaN(w) || isNaN(h) || h === 0 || w === 0) {
        alert("Width and height must be numbers; reverting to the old width and height.");
    } else {
        pw.w = w;
        pw.h = h;
        pw.render();
    }
    $('#width').val(pw.w);
    $('#height').val(pw.h);
}
$('#width, #height').change(pw.updateHW);

pw.dataTableGenerator = {
	uniqueProperties: [],
	table: $('<table class="dataset_table"><tr class="table_header"></tr></table>')
};

pw.dataTableGenerator.parseJSON = function(jsonObj) {
	for (var i = 0; i<jsonObj.length; i++) {
		for (prop in jsonObj[i]) {
			this.checkUniqueness(prop);
		}
	}
}

pw.dataTableGenerator.checkUniqueness = function(prop) {
	if (this.uniqueProperties.indexOf(prop) != -1) {
		return false;
	} else {
		this.uniqueProperties.push(prop);
		return true;
	}
}

pw.dataTableGenerator.writeTable = function(jsonObj) {
	var header = this.table.find('.table_header');
	for (var i=0; i < this.uniqueProperties.length; i++) {
		header.append('<th>' + this.uniqueProperties[i] + '</th>');
	};
	for (var k=0; k < jsonObj.length; k++) {
		var row = this.table.append('<tr/>');
		for (var j=0; j < this.uniqueProperties.length; j++) {
			row.append($('<td>' + jsonObj[this.uniqueProperties[j]] + '</td>'));
		}; 
	};
	return this.table;
};

// Generate a table of data sets
// Read in an array of objects
pw.dataTableGenerator.exampleJSON = [
	{
		color: "red",
		value: "#f00"
	},
	{
		color: "green",
		value: "#0f0"
	},
	{
		color: "blue",
		value: "#00f"
	},
	{
		color: "cyan",
		value: "#0ff",
		hue: "something"
	},
	{
		color: "magenta",
		value: "#f0f"
	},
	{
		color: "yellow",
		value: "#ff0"
	},
	{
		color: "black",
		value: "#000"
	}
];


pw.dataTableGenerator.parseJSON(pw.dataTableGenerator.exampleJSON);
/*
	TODO Fix dataTableGenerator and patch back in.
*/
// $('body').append(pw.dataTableGenerator.writeTable(pw.dataTableGenerator.exampleJSON));

/*
	TODO integrate dataTableGenerator with pw.dataSet -- preserve interface fieldList[] in pw.dataSetProperty
*/

pw.layersUI = {
	collapseWarning: "<span id='collapse-warning'>Collapse to reorder</span>",
	accordionOptions: {
		active: false,
		autoHeight: true,
		change: function(event, ui) {
			pw.layersUI.lockSortableOnOpenItem();
		},
		collapsible: true,
		fillSpace: false,
		header: '.header'
	},
	markContainer: [
		'<li>',
			'<div class="mark_container">',
				'<div class="handlebar">',
					'<span id="collapse-warning">Collapse to reorder</span>',
					'<div class="mark_controls">',
						'<div class="closer">delete</div>',
						'<div class="show_code">code</div>',
					'</div>', //end .mark_controls
				'</div>', // end .handlebar
				'<h3 class="header"><a href="#">Header text</a></h3>',
				'<div class="mark_properties"></div>',
				'<div class="code_flyout">',
				'<pre></pre>',
				'</div>', // end .code_flyout
			'</div>', // end .mark_container
		'</li>'
	].join(" "),
	
	beforeCode: ['<!DOCTYPE html>',
	'<html>',
	'<head>',
	'	<title>Your Protowiz visualization</title>',
	'	<script src="[[[INSERT PATH TO PROTOVIS.JS HERE]]]" type="text/javascript" charset="utf-8"></script>',
	'	<script type="text/javascript" charset="utf-8">',
	''].join("\n"),

	afterCode: ['',
	'',
	'	</script>',
	'</head>',
	'<body>',
	'	<div id="fig"></div>',
	'</body>',
	'</html>'].join("\n")
};

pw.layersUI.accordionHasChanged = function() {
	console.log("accordionHasChanged fires");
	this.lockSortableOnOpenItem();
	this.computeOrderOfMarks();
};

pw.layersUI.addMarkToList = function (spec) {
	var newContainer = $(this.markContainer);
	var mark = pw.mark(spec);
	var codeFlyout = newContainer.find('.code_flyout').dialog(
		{
			autoOpen: false,
			/*
				TODO try to insert the codeFlyout next to its mark...
			*/
			// position: [30,40]
		});
	newContainer.find(".mark_properties").append(mark.formHTML);
	newContainer.find('.closer').bind('click', this.deleteMarkFromList);
	newContainer.attr('id', mark.id);
	
	$('.show_code', newContainer).click(function() {
		console.log("show_code has been clicked");
		codeFlyout.find('pre').empty().text($.trim(mark.code()));
		codeFlyout.dialog('option', 'title', "Code for " + mark.properties.name());
		codeFlyout.dialog('option', 'width', '500px');
		codeFlyout.dialog('open');
	});
	
	// Initialize the header name with the type of mark
	newContainer.find('.header a').text(mark.properties.name());
	
	// Register an event handler to the nameProperty input field so that the accordion header gets updated in the event of a change
	newContainer.find('.nameProperty input').bind('change', function() {
		console.log("change in the nameProperty field has been detected");
		newContainer.find('.header a').text(mark.properties.name());
		pw.layersUI.writeInheritanceToHeader();
	});
	
	// Put panels into the panels list, all other marks into the marks list
	/*
		TODO Make this work with different accordions or whatever.
	*/
	if (mark.properties.type() === "Panel") {
		// newContainer.sortable('option', 'cancel');
		// newContainer.addClass('no-drag');
		$('#mark_list').prepend(newContainer);
	} else {
		$("#mark_list").append(newContainer);
	}

	this.renderMarksList();
	$('#accordion').accordion('option', 'active', ':last');
	$('input.nameProperty', newContainer).focus();
	pw.render();
	this.setCollapseWarning();
	return mark;
};

pw.layersUI.checkParentRelationship = function(event, ui) {
	console.log("checkParentRelationship fires");
	var theParent;
	var draggedMark = pw.marks.getObjectWithId(ui.item.attr('id'));
	
	// Check whether the dragged mark has a parent.
	if (draggedMark.getParent()) {
		theParent = draggedMark.getParent();
		// Look at each <li> element in turn...
		$('#mark_list li:not(.ui-sortable-placeholder)').each(function(index, element) {
			
			// ...and test whether the dragged element is on top of it.
			if (ui.position.top < $(element).position().top) {
				// Compare the id of the current <li> element to that of the dragged mark's parent's id
				if (theParent.id === parseInt($(element).attr('id'), 10)) {
					window.alert("Dragging this here will break a parent child relationship! Proceed?");
				} else {
					console.log("No cause for concern.");
				}
			};
		});
	} else {
		console.log("This has no parents.");
	};	
};

pw.layersUI.computeOrderOfMarks = function() {
	console.log("computeOrderOfMarks fires");
	// get the collection of <li> elements in the <ol>
	var marks = $('#mark_list li');
	// create an array of all ID's in order
	var markIDs = [];
	var tempMarks = [];
	marks.each(function() {
		markIDs.push($(this).attr('id'));
	})
	// Go through each id in the new array that reflects the order of elements in the DOM...
	for (var i=0; i < markIDs.length; i++) {
		// ...and compare the id with the id of the object in pw.marks.
		for (var j=0; j < pw.marks.length; j++) {
			// If we have a match...
			if (pw.marks[j].id.toString() === markIDs[i]) {
				// ...set the layer property of the matching mark to the length of tempMarks as it will always be its last element...
				pw.marks[j].layer = tempMarks.length;
				// ...and then push it onto the end of tempMarks, thus increasing its length and setting up the next element's layer number correctly.
				tempMarks.push(pw.marks[j]);
				break;
			};
		};
	};
	for (var k=0; k < tempMarks.length; k++) {
		// Replace the original order of pw.marks with the new order one by one		
		pw.marks.splice(k, 1, tempMarks[k])
	};
	pw.render();
};

pw.layersUI.deleteMarkFromList = function() {
	console.log("$(this).parents() in deleteMarkFromList:");
	console.log($(this).parents());
	var idToDelete = $(this).parents('li').attr('id');
	console.log("idToDelete is " + idToDelete);
	for (var i=0; i < pw.marks.length; i++) {
		if (idToDelete == pw.marks[i].id) {
			console.log("Found element in pw.marks with id " + pw.marks[i].id);
			pw.marks[i].del();
		}
	}
	// After deleting an element, make sure that no properties still list the deleted element
	for (var i=0; i < pw.marks.length; i++) {
			pw.marks[i].updateLists();
	}
	$(this).parents('li').remove();
	pw.layersUI.renderMarksList();
	pw.render();
};

pw.layersUI.lockSortableOnOpenItem = function(evt, ui) {
	console.log("pw.layersUI.lockSortableOnOpenItem fires");
	
	// Set the currently open accordion item to un-sortable...
	$('#mark_list').sortable('option', 'cancel', 'li:has(.ui-state-active)');
	
	this.setCollapseWarning();
}

pw.layersUI.refreshDatasetSelector = function() {
	console.log("call to pw.layersUI.refreshDatasetSelector");
	var datasets_selector = $('#datasets_selector');
	var selected = datasets_selector.val();
	var firstOption = $('<option selected disabled>Data sets</option>');
	datasets_selector.empty();
	// Insert the first element
	if (pw.dataSets.length === 0) {
		$('#datasets_selector').append(firstOption);
	}
	// Populate the select element in Scales with the existing scales
	for (var i=0; i < pw.dataSets.length; i++) {
		var optionElement = $('<option/>')
		optionElement.val(i).text(pw.dataSets[i].properties.name());
		if (i.toString() === selected.toString()) {
		    optionElement.attr('selected', 'selected');
		}
		$('#datasets_selector').append(optionElement);
	}		
}

pw.layersUI.refreshScaleSelector = function() {
	console.log("call to pw.layersUI.refreshScaleSelector");
	var scales_selector = $('#scales_selector');
	var selected = scales_selector.val();
	var firstOption = $('<option selected disabled>Scales</option>');
	scales_selector.empty();
	// Insert the first element
	if (pw.scales.length === 0) {
		$('#scales_selector').append(firstOption);
	}
	// Populate the select element in Scales with the existing scales
	for (var i=0; i < pw.scales.length; i++) {
		var optionElement = $('<option/>')
		optionElement.val(i).text(pw.scales[i].properties.name());
		if (i.toString() === selected.toString()) {
		    optionElement.attr('selected', 'selected');
		}
		$('#scales_selector').append(optionElement);
	}		
}

pw.layersUI.renderMarksList = function() {
	console.log("call to renderMarksList");
	$('#mark_list').sortable('refresh');
	$('#accordion').accordion('destroy');
	$('#accordion').accordion(this.accordionOptions);
}

pw.layersUI.setCollapseWarning = function() {
	// First, remove every occurrence of #collapse-warning if there is one...
	$('#collapse-warning').remove();
	// ...then for every handlebar (there should only ever be one...) contained by a mark_container 
	// that has a child with class .ui-state-active set (== is open) append the collapse warning
	$('.mark_container:has(.ui-state-active) > .handlebar').append(this.collapseWarning);
}

pw.layersUI.visualizeDraggingConstraints = function(event, ui) {
	var draggedID = ui.item.attr('id');
	var constrainingElements = [];
	
	// find parent of dragged element
	var obj = pw.marks.getObjectWithId(draggedID);
	if (obj.getParent) {
		constrainingElements.push(obj.getParent().id);
	};
	// find children of dragged element
	console.log("Chilluns:");
	console.log(obj.getChildren());
	$.each(obj.getChildren(), function(index, child) {
		constrainingElements.push(child.id);
	});
	// put their id's into an array
	console.log("constrainingElements: ");
	console.log(constrainingElements);
	
	// for each id, find the li element and color its handlebar red
	$(constrainingElements).each(function(index, id) {
		$('#' + id).find('.handlebar').addClass('constrainingElement');
	});
	
};

pw.layersUI.writeInheritanceToHeader = function() {
	$('#mark_list li').each(function(index, element) {
		try {
			var obj = pw.marks.getObjectWithId($(element).attr('id'));
			if (obj.getParent()) {
				var parentsName = obj.getParent().properties.name();
				$(element).find('h3>a').text(obj.properties.name() + ' < ' + parentsName);
			}
		} catch(e) {
			console.log("Error in writeInheritanceToHeader");
			console.log(e);
		}
	});
};
// Prepare the DOM for displaying the layers UI
$(function() {
	// Defines the accordion widget for the marks list.
	$('#accordion, #panels').accordion(pw.layersUI.accordionOptions);

	// Defines sortability on the marks list.
	$('#mark_list, #panels_list').sortable({
		axis: 'y',
		beforeStop: function(event, ui) {
			console.log("beforeStop fires");
			pw.layersUI.checkParentRelationship(event, ui);
		},
		// Prevent the open accordion item from being sorted
		cancel: ' li:has(.no-drag), li:has(.ui-state-active)',
		change: function(event, ui) {
			console.log("change fires");
		},
		containment: '#accordion',
		cursor: 'crosshair',
		forcePlaceholderSize: true,
		handle: '.handlebar',
		items: 'li:not(.no-drag)',
		start: function(event, ui) {
			pw.layersUI.visualizeDraggingConstraints(event, ui);
		},
		stop: function(event, ui) {
			$('.handlebar').removeClass('constrainingElement');
		},
		update: function() {
			console.log("update fires");
			pw.layersUI.computeOrderOfMarks();
			pw.marks.updateLists();
		}
	});

	// Attach event listeners to the mark type buttons
	$('#mark_creators input').click(function(eventData) {
		// determine type of button that's been clicked
		var type = eventData.currentTarget.name;
		pw.layersUI.addMarkToList({type: type, data: ''});
	});
	
	// Data set selector change event listener
	$('#datasets_selector').change(function(eventData) {
		console.log("#datasets_selector has changed");
		$('#dataSets .mark').show();
		var selected = $('#datasets_selector option:selected');
		var datasetForms = $('#dataSets .nameProperty input');
		// For each input field
		datasetForms.each(function() {
			console.log(selected[0].value +  ', ' + this.value);
			if (this.value !== pw.dataSets[selected[0].value].properties.name()) {
				$(this).parents('.mark').hide();
			}
		});
	});
	
	// Scales selector change event listener
	$('#scales_selector').change(function(eventData) {
		console.log("#scales_selector has changed");
		$('#scales .mark').show();
		var selected = $('#scales_selector option:selected');
		var scaleForms = $('#scales .nameProperty input');
		// For each input field
		scaleForms.each(function() {
			console.log(selected[0].value +  ', ' + this.value);
			if (this.value !== pw.scales[selected[0].value].properties.name()) {
				$(this).parents('.mark').hide();
			}
		});
	});
	
	// New data set button event listener
	$('#new_dataset').click(function() {
		console.log("New dataset has been created");
		$( "#new_dataset_form" ).dialog('open');
	});
	
	// New data set form
	$('#new_dataset_form').dialog({
		autoOpen: false,
		height: 500,
		width: 500,
		modal: true,
		buttons: {
			"Cancel" : function() {
				$('*').removeClass("ui-state-error");
				$(this).dialog('close');
			},  // end Cancel button
			
			"Create data set": function() {
				if ($('#new_dataset_name').val() && $('#dataset_code').val()) {
					$('*').removeClass("ui-state-error");
					var newDataset = pw.dataSet({
						name: $('#new_dataset_name').val(),
						dataSet: $.parseJSON($('#dataset_code').val())
					});
					$('#datasets_selector option:last').attr('selected', 'selected');
					$('#datasets_selector').trigger('change');
					pw.dataSets.updateLists();
					$(this).dialog('close');
				} else {
					$('.validateTips').addClass("ui-state-error");
					$('#new_dataset_name').addClass('ui-state-error');
				}
			} // end Create data set button
		}
	});
	
	// New scale button event listener
	$('#new_scale').click(function() {
		console.log("New scale has been created");
		$( "#new_scale_form" ).dialog('open');
	});
	
	// New scale form
	$('#new_scale_form').dialog({
		autoOpen: false,
		height: 180,
		width: 250,
		modal: true,
		buttons: {
			"Cancel" : function() {
				$('*').removeClass("ui-state-error");
				$(this).dialog('close');
			},
			
			"Create scale": function() {
				if ($('#new_scale_name').val()) {
					$('*').removeClass("ui-state-error");
					var newScale = pw.scale({
						name: $('#new_scale_name').val()
					});
					console.log(newScale.formHTML.find('.mark'));
					newScale.formHTML.find('.mark').append('<div class="closer">delete</div>');
					$('#scales_selector option:last').attr('selected', 'selected');
					$('#scales_selector').trigger('change');
					$(this).dialog('close');
					pw.render();
				} else {
					$('.validateTips').addClass("ui-state-error");
					$('#new_scale_name').addClass('ui-state-error');
				}
					
			}
		}
	});
	
	$('#show_pv_code').click(function() {
		$('#code_dialog').dialog('open');
	});
	
	$('#code_dialog').dialog({
		autoOpen: false,
		height: 600,
		width: 600,
		modal: true
	})
		
	$('#advanced').change(function() {
	    pw.showAdvancedProperties = $(this).is(":checked");
	    $.each(pw.marks, function(i, v) {
	       v.updateVisibility(); 
	    });
	});
	
	pw.layersUI.refreshScaleSelector();
	pw.layersUI.refreshDatasetSelector();
	
	// Show only one scale at startup.
	$('#scales .mark:gt(0)').each(function() {
		$(this).hide();
	});
	
	// Show only one data set at startup.
	$('#dataSets .mark:gt(0)').each(function() {
		$(this).hide();
	});
	
	pw.layersUI.renderMarksList();
	pw.layersUI.writeInheritanceToHeader();
	
}); // END jQuery init

// Test it out
pw.testdata = pv.range(100).map(function(x) {
    return {x: x, y: Math.random(), z: Math.pow(10, 2 * Math.random())};
});
s = pw.scale({
    type: 'linear',
    kind: 'spatial',
    name: 'x',
    domain: '0, 99', // setting a log scale with 0 at the bottom of the range locks up the page; we'll sidestep that for now
    range: '0, w'
});
s2 = pw.scale({
    type: 'linear',
    kind: 'spatial',
    name: 'y',
    domain: '0, 1', // setting a log scale with 0 at the bottom of the range locks up the page; we'll sidestep that for now
    range: '0, h'
});
s3 = pw.scale({
    type: 'log',
    kind: 'color',
    name: 'c',
    domain: '1, 100', // setting a log scale with 0 at the bottom of the range locks up the page; we'll sidestep that for now
    range: "'#ff7f0e', '#8c564b'"
});
randomData = '[' + pv.range(100).map(function(x) {return "{x: " + x + ", y: " + Math.random() + ", z: " + Math.pow(10, 2 * Math.random()) + "}";}).join(', ') + ']';
d = pw.dataSet({
   name: 'demo',
   dataSet: randomData
});
d2 = pw.dataSet({
   name: 'driving',
   dataSet: '[  {side: "left", year: 1956, miles: 3675, gas: 2.38},  {side: "right", year: 1957, miles: 3706, gas: 2.40},  {side: "bottom", year: 1958, miles: 3766, gas: 2.26},  {side: "top", year: 1959, miles: 3905, gas: 2.31},  {side: "right", year: 1960, miles: 3935, gas: 2.27},  {side: "bottom", year: 1961, miles: 3977, gas: 2.25},  {side: "right", year: 1962, miles: 4085, gas: 2.22},  {side: "bottom", year: 1963, miles: 4218, gas: 2.12},  {side: "bottom", year: 1964, miles: 4369, gas: 2.11},  {side: "bottom", year: 1965, miles: 4538, gas: 2.14},  {side: "top", year: 1966, miles: 4676, gas: 2.14},  {side: "bottom", year: 1967, miles: 4827, gas: 2.14},  {side: "right", year: 1968, miles: 5038, gas: 2.13},  {side: "right", year: 1969, miles: 5207, gas: 2.07},  {side: "right", year: 1970, miles: 5376, gas: 2.01},  {side: "bottom", year: 1971, miles: 5617, gas: 1.93},  {side: "bottom", year: 1972, miles: 5973, gas: 1.87},  {side: "right", year: 1973, miles: 6154, gas: 1.90},  {side: "left", year: 1974, miles: 5943, gas: 2.34},  {side: "bottom", year: 1975, miles: 6111, gas: 2.31},  {side: "bottom", year: 1976, miles: 6389, gas: 2.32},  {side: "right", year: 1977, miles: 6630, gas: 2.36},  {side: "bottom", year: 1978, miles: 6883, gas: 2.23},  {side: "left", year: 1979, miles: 6744, gas: 2.68},  {side: "left", year: 1980, miles: 6672, gas: 3.30},  {side: "right", year: 1981, miles: 6732, gas: 3.30},  {side: "right", year: 1982, miles: 6835, gas: 2.92},  {side: "right", year: 1983, miles: 6943, gas: 2.66},  {side: "right", year: 1984, miles: 7130, gas: 2.48},  {side: "right", year: 1985, miles: 7323, gas: 2.36},  {side: "left", year: 1986, miles: 7558, gas: 1.76},  {side: "top", year: 1987, miles: 7770, gas: 1.76},  {side: "bottom", year: 1988, miles: 8089, gas: 1.68},  {side: "left", year: 1989, miles: 8397, gas: 1.75},  {side: "top", year: 1990, miles: 8529, gas: 1.88},  {side: "right", year: 1991, miles: 8535, gas: 1.78},  {side: "right", year: 1992, miles: 8662, gas: 1.69},  {side: "left", year: 1993, miles: 8855, gas: 1.60},  {side: "bottom", year: 1994, miles: 8909, gas: 1.59},  {side: "bottom", year: 1995, miles: 9150, gas: 1.60},  {side: "top", year: 1996, miles: 9192, gas: 1.67},  {side: "right", year: 1997, miles: 9416, gas: 1.65},  {side: "bottom", year: 1998, miles: 9590, gas: 1.39},  {side: "right", year: 1999, miles: 9687, gas: 1.50},  {side: "top", year: 2000, miles: 9717, gas: 1.89},  {side: "left", year: 2001, miles: 9699, gas: 1.77},  {side: "bottom", year: 2002, miles: 9814, gas: 1.64},  {side: "right", year: 2003, miles: 9868, gas: 1.86},  {side: "left", year: 2004, miles: 9994, gas: 2.14},  {side: "left", year: 2005, miles: 10067, gas: 2.53},  {side: "right", year: 2006, miles: 10037, gas: 2.79},  {side: "right", year: 2007, miles: 10025, gas: 2.95},  {side: "left", year: 2008, miles: 9880, gas: 3.31},  {side: "bottom", year: 2009, miles: 9657, gas: 2.38},  {side: "left", year: 2010, miles: 9596, gas: 2.61}]' 
});
d3 = pw.dataSet({
   name: 'wheat',
   dataSet: '[{ year: 1565, wheat: 41, wages: 5 },  { year: 1570, wheat: 45, wages: 5.05 },  { year: 1575, wheat: 42, wages: 5.08 },  { year: 1580, wheat: 49, wages: 5.12 },  { year: 1585, wheat: 41.5, wages: 5.15 },  { year: 1590, wheat: 47, wages: 5.25 },  { year: 1595, wheat: 64, wages: 5.54 },  { year: 1600, wheat: 27, wages: 5.61 },  { year: 1605, wheat: 33, wages: 5.69 },  { year: 1610, wheat: 32, wages: 5.78 },  { year: 1615, wheat: 33, wages: 5.94 },  { year: 1620, wheat: 35, wages: 6.01 },  { year: 1625, wheat: 33, wages: 6.12 },  { year: 1630, wheat: 45, wages: 6.22 },  { year: 1635, wheat: 33, wages: 6.3 },  { year: 1640, wheat: 39, wages: 6.37 },  { year: 1645, wheat: 53, wages: 6.45 },  { year: 1650, wheat: 42, wages: 6.5 },  { year: 1655, wheat: 40.5, wages: 6.6 },  { year: 1660, wheat: 46.5, wages: 6.75 },  { year: 1665, wheat: 32, wages: 6.8 },  { year: 1670, wheat: 37, wages: 6.9 },  { year: 1675, wheat: 43, wages: 7 },  { year: 1680, wheat: 35, wages: 7.3 },  { year: 1685, wheat: 27, wages: 7.6 },  { year: 1690, wheat: 40, wages: 8 },  { year: 1695, wheat: 50, wages: 8.5 },  { year: 1700, wheat: 30, wages: 9 },  { year: 1705, wheat: 32, wages: 10 },  { year: 1710, wheat: 44, wages: 11 },  { year: 1715, wheat: 33, wages: 11.75 },  { year: 1720, wheat: 29, wages: 12.5 },  { year: 1725, wheat: 39, wages: 13 },  { year: 1730, wheat: 26, wages: 13.3 },  { year: 1735, wheat: 32, wages: 13.6 },  { year: 1740, wheat: 27, wages: 14 },  { year: 1745, wheat: 27.5, wages: 14.5 },  { year: 1750, wheat: 31, wages: 15 },  { year: 1755, wheat: 35.5, wages: 15.7 },  { year: 1760, wheat: 31, wages: 16.5 },  { year: 1765, wheat: 43, wages: 17.6 },  { year: 1770, wheat: 47, wages: 18.5 },  { year: 1775, wheat: 44, wages: 19.5 },  { year: 1780, wheat: 46, wages: 21 },  { year: 1785, wheat: 42, wages: 23 },  { year: 1790, wheat: 47.5, wages: 25.5 },  { year: 1795, wheat: 76, wages: 27.5 },  { year: 1800, wheat: 79, wages: 28.5 },  { year: 1805, wheat: 81, wages: 29.5 },  { year: 1810, wheat: 99, wages: 30 },  { year: 1815, wheat: 78 }, { year: 1820, wheat: 54 },  { year: 1821, wheat: 54 }];'
});
d4 = pw.dataSet({
   name: 'monarch',
   dataSet: '[{ name: "Elizabeth", start: 1565, end: 1603 },  { name: "James I", start: 1603, end: 1625 },  { name: "Charles I", start: 1625, end: 1649 },  { name: "Cromwell", start: 1649, end: 1660, commonwealth: true },  { name: "Charles II", start: 1660, end: 1685 },  { name: "James II", start: 1685, end: 1689 },  { name: "W&M", start: 1689, end: 1702 },  { name: "Anne", start: 1702, end: 1714 },  { name: "George I", start: 1714, end: 1727 },  { name: "George II", start: 1727, end: 1760 },  { name: "George III", start: 1760, end: 1820 },  { name: "George IV", start: 1820, end: 1821 }]' 
});
d5 = pw.dataSet({
   name: 'barley',
   dataSet: '[{ yield: 27.00000, variety: "Manchuria", year: 1931, site: "University Farm" },  { yield: 48.86667, variety: "Manchuria", year: 1931, site: "Waseca" },  { yield: 27.43334, variety: "Manchuria", year: 1931, site: "Morris" },  { yield: 39.93333, variety: "Manchuria", year: 1931, site: "Crookston" },  { yield: 32.96667, variety: "Manchuria", year: 1931, site: "Grand Rapids" },  { yield: 28.96667, variety: "Manchuria", year: 1931, site: "Duluth" },  { yield: 43.06666, variety: "Glabron", year: 1931, site: "University Farm" },  { yield: 55.20000, variety: "Glabron", year: 1931, site: "Waseca" },  { yield: 28.76667, variety: "Glabron", year: 1931, site: "Morris" },  { yield: 38.13333, variety: "Glabron", year: 1931, site: "Crookston" },  { yield: 29.13333, variety: "Glabron", year: 1931, site: "Grand Rapids" },  { yield: 29.66667, variety: "Glabron", year: 1931, site: "Duluth" },  { yield: 35.13333, variety: "Svansota", year: 1931, site: "University Farm" },  { yield: 47.33333, variety: "Svansota", year: 1931, site: "Waseca" },  { yield: 25.76667, variety: "Svansota", year: 1931, site: "Morris" },  { yield: 40.46667, variety: "Svansota", year: 1931, site: "Crookston" },  { yield: 29.66667, variety: "Svansota", year: 1931, site: "Grand Rapids" },  { yield: 25.70000, variety: "Svansota", year: 1931, site: "Duluth" },  { yield: 39.90000, variety: "Velvet", year: 1931, site: "University Farm" },  { yield: 50.23333, variety: "Velvet", year: 1931, site: "Waseca" },  { yield: 26.13333, variety: "Velvet", year: 1931, site: "Morris" },  { yield: 41.33333, variety: "Velvet", year: 1931, site: "Crookston" },  { yield: 23.03333, variety: "Velvet", year: 1931, site: "Grand Rapids" },  { yield: 26.30000, variety: "Velvet", year: 1931, site: "Duluth" },  { yield: 36.56666, variety: "Trebi", year: 1931, site: "University Farm" },  { yield: 63.83330, variety: "Trebi", year: 1931, site: "Waseca" },  { yield: 43.76667, variety: "Trebi", year: 1931, site: "Morris" },  { yield: 46.93333, variety: "Trebi", year: 1931, site: "Crookston" },  { yield: 29.76667, variety: "Trebi", year: 1931, site: "Grand Rapids" },  { yield: 33.93333, variety: "Trebi", year: 1931, site: "Duluth" },  { yield: 43.26667, variety: "No. 457", year: 1931, site: "University Farm" },  { yield: 58.10000, variety: "No. 457", year: 1931, site: "Waseca" },  { yield: 28.70000, variety: "No. 457", year: 1931, site: "Morris" },  { yield: 45.66667, variety: "No. 457", year: 1931, site: "Crookston" },  { yield: 32.16667, variety: "No. 457", year: 1931, site: "Grand Rapids" },  { yield: 33.60000, variety: "No. 457", year: 1931, site: "Duluth" },  { yield: 36.60000, variety: "No. 462", year: 1931, site: "University Farm" },  { yield: 65.76670, variety: "No. 462", year: 1931, site: "Waseca" },  { yield: 30.36667, variety: "No. 462", year: 1931, site: "Morris" },  { yield: 48.56666, variety: "No. 462", year: 1931, site: "Crookston" },  { yield: 24.93334, variety: "No. 462", year: 1931, site: "Grand Rapids" },  { yield: 28.10000, variety: "No. 462", year: 1931, site: "Duluth" },  { yield: 32.76667, variety: "Peatland", year: 1931, site: "University Farm" },  { yield: 48.56666, variety: "Peatland", year: 1931, site: "Waseca" },  { yield: 29.86667, variety: "Peatland", year: 1931, site: "Morris" },  { yield: 41.60000, variety: "Peatland", year: 1931, site: "Crookston" },  { yield: 34.70000, variety: "Peatland", year: 1931, site: "Grand Rapids" },  { yield: 32.00000, variety: "Peatland", year: 1931, site: "Duluth" },  { yield: 24.66667, variety: "No. 475", year: 1931, site: "University Farm" },  { yield: 46.76667, variety: "No. 475", year: 1931, site: "Waseca" },  { yield: 22.60000, variety: "No. 475", year: 1931, site: "Morris" },  { yield: 44.10000, variety: "No. 475", year: 1931, site: "Crookston" },  { yield: 19.70000, variety: "No. 475", year: 1931, site: "Grand Rapids" },  { yield: 33.06666, variety: "No. 475", year: 1931, site: "Duluth" },  { yield: 39.30000, variety: "Wisconsin No. 38", year: 1931, site: "University Farm" },  { yield: 58.80000, variety: "Wisconsin No. 38", year: 1931, site: "Waseca" },  { yield: 29.46667, variety: "Wisconsin No. 38", year: 1931, site: "Morris" },  { yield: 49.86667, variety: "Wisconsin No. 38", year: 1931, site: "Crookston" },  { yield: 34.46667, variety: "Wisconsin No. 38", year: 1931, site: "Grand Rapids" },  { yield: 31.60000, variety: "Wisconsin No. 38", year: 1931, site: "Duluth" },  { yield: 26.90000, variety: "Manchuria", year: 1932, site: "University Farm" },  { yield: 33.46667, variety: "Manchuria", year: 1932, site: "Waseca" },  { yield: 34.36666, variety: "Manchuria", year: 1932, site: "Morris" },  { yield: 32.96667, variety: "Manchuria", year: 1932, site: "Crookston" },  { yield: 22.13333, variety: "Manchuria", year: 1932, site: "Grand Rapids" },  { yield: 22.56667, variety: "Manchuria", year: 1932, site: "Duluth" },  { yield: 36.80000, variety: "Glabron", year: 1932, site: "University Farm" },  { yield: 37.73333, variety: "Glabron", year: 1932, site: "Waseca" },  { yield: 35.13333, variety: "Glabron", year: 1932, site: "Morris" },  { yield: 26.16667, variety: "Glabron", year: 1932, site: "Crookston" },  { yield: 14.43333, variety: "Glabron", year: 1932, site: "Grand Rapids" },  { yield: 25.86667, variety: "Glabron", year: 1932, site: "Duluth" },  { yield: 27.43334, variety: "Svansota", year: 1932, site: "University Farm" },  { yield: 38.50000, variety: "Svansota", year: 1932, site: "Waseca" },  { yield: 35.03333, variety: "Svansota", year: 1932, site: "Morris" },  { yield: 20.63333, variety: "Svansota", year: 1932, site: "Crookston" },  { yield: 16.63333, variety: "Svansota", year: 1932, site: "Grand Rapids" },  { yield: 22.23333, variety: "Svansota", year: 1932, site: "Duluth" },  { yield: 26.80000, variety: "Velvet", year: 1932, site: "University Farm" },  { yield: 37.40000, variety: "Velvet", year: 1932, site: "Waseca" },  { yield: 38.83333, variety: "Velvet", year: 1932, site: "Morris" },  { yield: 32.06666, variety: "Velvet", year: 1932, site: "Crookston" },  { yield: 32.23333, variety: "Velvet", year: 1932, site: "Grand Rapids" },  { yield: 22.46667, variety: "Velvet", year: 1932, site: "Duluth" },  { yield: 29.06667, variety: "Trebi", year: 1932, site: "University Farm" },  { yield: 49.23330, variety: "Trebi", year: 1932, site: "Waseca" },  { yield: 46.63333, variety: "Trebi", year: 1932, site: "Morris" },  { yield: 41.83333, variety: "Trebi", year: 1932, site: "Crookston" },  { yield: 20.63333, variety: "Trebi", year: 1932, site: "Grand Rapids" },  { yield: 30.60000, variety: "Trebi", year: 1932, site: "Duluth" },  { yield: 26.43334, variety: "No. 457", year: 1932, site: "University Farm" },  { yield: 42.20000, variety: "No. 457", year: 1932, site: "Waseca" },  { yield: 43.53334, variety: "No. 457", year: 1932, site: "Morris" },  { yield: 34.33333, variety: "No. 457", year: 1932, site: "Crookston" },  { yield: 19.46667, variety: "No. 457", year: 1932, site: "Grand Rapids" },  { yield: 22.70000, variety: "No. 457", year: 1932, site: "Duluth" },  { yield: 25.56667, variety: "No. 462", year: 1932, site: "University Farm" },  { yield: 44.70000, variety: "No. 462", year: 1932, site: "Waseca" },  { yield: 47.00000, variety: "No. 462", year: 1932, site: "Morris" },  { yield: 30.53333, variety: "No. 462", year: 1932, site: "Crookston" },  { yield: 19.90000, variety: "No. 462", year: 1932, site: "Grand Rapids" },  { yield: 22.50000, variety: "No. 462", year: 1932, site: "Duluth" },  { yield: 28.06667, variety: "Peatland", year: 1932, site: "University Farm" },  { yield: 36.03333, variety: "Peatland", year: 1932, site: "Waseca" },  { yield: 43.20000, variety: "Peatland", year: 1932, site: "Morris" },  { yield: 25.23333, variety: "Peatland", year: 1932, site: "Crookston" },  { yield: 26.76667, variety: "Peatland", year: 1932, site: "Grand Rapids" },  { yield: 31.36667, variety: "Peatland", year: 1932, site: "Duluth" },  { yield: 30.00000, variety: "No. 475", year: 1932, site: "University Farm" },  { yield: 41.26667, variety: "No. 475", year: 1932, site: "Waseca" },  { yield: 44.23333, variety: "No. 475", year: 1932, site: "Morris" },  { yield: 32.13333, variety: "No. 475", year: 1932, site: "Crookston" },  { yield: 15.23333, variety: "No. 475", year: 1932, site: "Grand Rapids" },  { yield: 27.36667, variety: "No. 475", year: 1932, site: "Duluth" },  { yield: 38.00000, variety: "Wisconsin No. 38", year: 1932, site: "University Farm" },  { yield: 58.16667, variety: "Wisconsin No. 38", year: 1932, site: "Waseca" },  { yield: 47.16667, variety: "Wisconsin No. 38", year: 1932, site: "Morris" },  { yield: 35.90000, variety: "Wisconsin No. 38", year: 1932, site: "Crookston" },  { yield: 20.66667, variety: "Wisconsin No. 38", year: 1932, site: "Grand Rapids" },  { yield: 29.33333, variety: "Wisconsin No. 38", year: 1932, site: "Duluth" }]' 
});
r = pw.layersUI.addMarkToList({
    type: 'Panel',
    name: 'root',
    x: {side: 'left', scale: '', field: '20'},
    y: {side: 'bottom', scale: '', field: '20'},
    height: {scale: '', field: 'h'},
    width: {scale: '', field: 'w'}
});
x_ax = pw.layersUI.addMarkToList({
    type: 'Rule',
    name: 'x_axis',
    parent: '0',
    data: 'scale0',
    x: {side: 'left', scale: '0', field: ''},
    strokeStyle: {scale: '', field: "'lightgray'", alpha: ''},
    tickLabel: "'bottom'"
});
y_ax = pw.layersUI.addMarkToList({
    type: 'Rule',
    name: 'y_axis',
    parent: '0',
    data: 'scale1',
    y: {side: 'bottom', scale: '1', field: ''},
    strokeStyle: {scale: '', field: "'lightgray'", alpha: ''},
    tickLabel: "'left'"
});
m = pw.layersUI.addMarkToList({
	type: 'Dot',
    name: 'exampleMark',
    parent: '0',
    data: '0',
    x: {side: 'left', field: 'd.x', scale: '0'},
    y: {side: 'bottom', field: 'd.y', scale: '1'},
    strokeStyle: {field: 'd.z', scale: '2'}
});
// m2 = pw.mark({
//     type: 'Label',
//     name: 'mark2',
//     parent: '0',
//     text: 'd.z.toFixed()'
// });
// m3 = pw.mark({
//     type: 'Wedge',
//     name: 'mark3',
//     parent: '1',
//     innerRadius: 8,
//     outerRadius: 10,
//     angle: 'Math.PI / 4'
// });

pw.dataSets.updateLists();
pw.render();

// Disable using the backspace key as a "back" button in the browser;
// otherwise, it's too easy to accidentally hit backspace outside of a form control
// and lose all your work.  This is only tested in Chrome.
$(window).keydown(function(event) {
	if (event.target.tagName !== 'INPUT' && event.which === 8) {
        event.preventDefault();
	}
});