// Add some helper funcitons to String from jquery.string
$.string(String.prototype);

// Add some helpers of our own to help with user-created identifiers
String.prototype.isReserved = function() {
    var reserved = {
        // official (or anticpated future) reserved words
        "abstract": true,
        "as": true,
        "boolean": true,
        "break": true,
        "byte": true,
        "case": true,
        "catch": true,
        "char": true,
        "class": true,
        "continue": true,
        "const": true,
        "debugger": true,
        "default": true,
        "delete": true,
        "do": true,
        "double": true,
        "else": true,
        "enum": true,
        "export": true,
        "extends": true,
        "false": true,
        "final": true,
        "finally": true,
        "float": true,
        "for": true,
        "function": true,
        "goto": true,
        "if": true,
        "implements": true,
        "import": true,
        "in": true,
        "instanceof": true,
        "int": true,
        "interface": true,
        "is": true,
        "long": true,
        "namespace": true,
        "native": true,
        "new": true,
        "null": true,
        "package": true,
        "private": true,
        "protected": true,
        "public": true,
        "return": true,
        "short": true,
        "static": true,
        "super": true,
        "switch": true,
        "synchronized": true,
        "this": true,
        "throw": true,
        "throws": true,
        "transient": true,
        "true": true,
        "try": true,
        "typeof": true,
        "use": true,
        "var": true,
        "void": true,
        "volatile": true,
        "while": true,
        "with": true,

        // words that probably should be reserved, at least in some implementations
        "alert": true,
        "frames": true,
        "outerHeight": true,
        "all": true,
        "frameRate": true,
        "outerWidth": true,
        "anchor": true,
        "function": true,
        "packages": true,
        "anchors": true,
        "getClass": true,
        "pageXOffset": true,
        "area": true,
        "hasOwnProperty": true,
        "pageYOffset": true,
        "Array": true,
        "hidden": true,
        "parent": true,
        "assign": true,
        "history": true,
        "parseFloat": true,
        "blur": true,
        "image": true,
        "parseInt": true,
        "button": true,
        "images": true,
        "password": true,
        "checkbox": true,
        "Infinity": true,
        "pkcs11": true,
        "clearInterval": true,
        "isFinite": true,
        "plugin": true,
        "clearTimeout": true,
        "isNaN": true,
        "prompt": true,
        "clientInformation": true,
        "isPrototypeOf": true,
        "propertyIsEnum": true,
        "close": true,
        "java": true,
        "prototype": true,
        "closed": true,
        "JavaArray": true,
        "radio": true,
        "confirm": true,
        "JavaClass": true,
        "reset": true,
        "constructor": true,
        "JavaObject": true,
        "screenX": true,
        "crypto": true,
        "JavaPackage": true,
        "screenY": true,
        "Date": true,
        "innerHeight": true,
        "scroll": true,
        "decodeURI": true,
        "innerWidth": true,
        "secure": true,
        "decodeURIComponent": true,
        "layer": true,
        "select": true,
        "defaultStatus": true,
        "layers": true,
        "self": true,
        "document": true,
        "length": true,
        "setInterval": true,
        "element": true,
        "link": true,
        "setTimeout": true,
        "elements": true,
        "location": true,
        "status": true,
        "embed": true,
        "Math": true,
        "String": true,
        "embeds": true,
        "mimeTypes": true,
        "submit": true,
        "encodeURI": true,
        "name": true,
        "taint": true,
        "encodeURIComponent": true,
        "NaN": true,
        "text": true,
        "escape": true,
        "navigate": true,
        "textarea": true,
        "eval": true,
        "navigator": true,
        "top": true,
        "event": true,
        "Number": true,
        "toString": true,
        "fileUpload": true,
        "Object": true,
        "undefined": true,
        "focus": true,
        "offscreenBuffering": true,
        "unescape": true,
        "form": true,
        "open": true,
        "untaint": true,
        "forms": true,
        "opener": true,
        "valueOf": true,
        "frame": true,
        "option": true,
        "window": true,
        "onbeforeunload": true,
        "ondragdrop": true,
        "onkeyup": true,
        "onmouseover": true,
        "onblur": true,
        "onerror": true,
        "onload": true,
        "onmouseup": true,
        "ondragdrop": true,
        "onfocus": true,
        "onmousedown": true,
        "onreset": true,
        "onclick": true,
        "onkeydown": true,
        "onmousemove": true,
        "onsubmit": true,
        "oncontextmenu": true,
        "onkeypress": true,
        "onmouseout": true,
        "onunload": true,
        
        // protowiz-specific reserved words
        "pw": true,
        "pv": true,
        "$": true,
        "jQuery": true
    };
    
    return reserved[this];
}

String.prototype.isName = function() {
    return /^[$A-Za-z_][$0-9A-Za-z_]*$/.exec(this);
}

// strip out all the stuff that would make this string not a valid identifier
// if this results in an empty string, return a default name (which can optionally be passed in)
// if instead it results in a reserved word, tack a character on the end to dodge the reserved word
String.prototype.toName = function(fallback) {
    var n = '';    
    if (this.isName()) {
        // Really?  "this" is an object, not a string?  Yes.
        n = this.toString();
    } else {
        n = this.replace(/\s+/g, '_').replace(/[^$0-9A-Za-z_]/g, '').replace(/^[0-9]+/, '');
    }

    if (n === '') {
        return (fallback.toName() || "default");
    } else {
        return n + (n.isReserved() ? '0' : '');
    }
}